/*
 * Copyright 2013 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.annotations.processor;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.TypeVariable;
import java.util.Date;
import java.util.Locale;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.Util;
import olg.csv.bean.annotations.DateProperty;
import olg.csv.bean.annotations.Param;
import olg.csv.bean.annotations.Parser;
import olg.csv.bean.parser.AbstractParser;
import olg.csv.bean.parser.ParseException;
import olg.csv.bean.parser.impl.DateParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class is responsible for processing the @Parser annotation.
 * 
 * @author Olivier Godineau
 * 
 */
public final class ParserProcessor {
	/**
	 * The class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(ParserProcessor.class);

	/**
	 * The Singleton instance.
	 */
	private static ParserProcessor instance = new ParserProcessor();

	/**
	 * Private constructor to ensure the existence of singleton.
	 */
	private ParserProcessor() {

	}
	/**
	 * The singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static ParserProcessor getInstance() {
		return instance;
	}

	/**
	 * Return the parser to use on a field. If no parser annotation is setted on
	 * the field, try to identify a default parser from the concrete class.
	 * 
	 * @param field
	 *            the field
	 * @param fieldConcreteClass
	 *            the concrete class from which instanciate the field values.
	 * @param <T>
	 *            the type the parser must return
	 * 
	 * @return the parser
	 * @throws AnnotationProcessorException
	 *             if an error occurs during annotation reading or if no parser
	 *             has been found.
	 * @see AbstractParser#identifyDefaultParser(Class)
	 */
	public <T> AbstractParser<T> getParser(Field field, Class<T> fieldConcreteClass)
			throws AnnotationProcessorException {
		AbstractParser<T> parser = null;

		Parser parserAnnotation = field.getAnnotation(Parser.class);
		if (parserAnnotation != null) {
			Class<? extends olg.csv.bean.parser.AbstractParser<?>> clazz = parserAnnotation.value();
			if (clazz.equals(olg.csv.bean.parser.impl.DateParser.class)) {
				parser = (AbstractParser<T>) getDateParser(field, parserAnnotation, fieldConcreteClass);
			} else {

				parser = getCustomParser(field, parserAnnotation, fieldConcreteClass);
			}
		} else {
			try {
				// Identifier le parser par défaut;
				parser = AbstractParser.identifyDefaultParser(fieldConcreteClass);
			} catch (UsageInvalideException ex) {
				LOGGER.error("{}@Column : can not identify a default {} parser : May be, use @Parser annotation ...",
						field.getName(), fieldConcreteClass.getSimpleName());
				throw new AnnotationProcessorException(ex);
			}
		}

		return parser;

	}

	/**
	 * Returns a date parser identified from the parser annotation setted on a
	 * field.
	 * 
	 * @param field
	 *            the field.
	 * @param parserAnnotation
	 *            the annotation setted on the field
	 * @param fieldConcreteClass
	 *            the concrete class from which instanciate the field values.
	 * @param <T>
	 *            the type the parser must return
	 * @return the date parser
	 * @throws AnnotationProcessorException
	 *             if an error occurs
	 */
	private <T> AbstractParser<Date> getDateParser(Field field, Parser parserAnnotation, Class<T> fieldConcreteClass)
			throws AnnotationProcessorException {
		AbstractParser<Date> parser = null;
		String format = null;
		Locale locale = null;
		Class<? extends Date> dateClass = null;
		boolean errorOccurs = false;

		try {
			dateClass = fieldConcreteClass.asSubclass(Date.class);
		} catch (ClassCastException ex) {
			errorOccurs = true;
			LOGGER.error("{}@Parser : {} field class must extend Date class to instanciate a DateParser",
					field.getName(), fieldConcreteClass.getSimpleName());
		}

		if (!Util.isConcrete(fieldConcreteClass)) {
			errorOccurs = true;
			LOGGER.error("{}@Parser : {} field class must be a concrete class", field.getName(),
					fieldConcreteClass.getSimpleName());
		}

		for (Param param : parserAnnotation.params()) {
			if (param.name().equals(DateProperty.FORMAT)) {
				format = param.value();
			} else {
				if (param.name().equals(DateProperty.LOCALE)) {
					locale = olg.csv.bean.loader.Util.getLocale(param.value());
					if (locale == null) {
						LOGGER.warn("{}@Parser : {} Locale property is not an available Locale, it will be ignored",
								field.getName(), param.value());
					}
				} else {
					LOGGER.warn("{}@Parser : {} is not a DateParser legal property, it will be ignored ",
							field.getName(), param.name());
				}
			}
		}

		if (format == null) { // Format is required
			errorOccurs = true;
			LOGGER.error("{}@Parser : DateParser {} property is required", field.getName(), DateProperty.FORMAT);
		} else {
			try {
				parser = new DateParser(format, dateClass, locale);

			} catch (UsageInvalideException ex) {
				errorOccurs = true;
				LOGGER.error(String.format("%s@Parser : DateParser cannot be instanciated", field.getName()), ex);
			}
		}

		if (errorOccurs) {
			throw new AnnotationProcessorException();
		}
		return parser;
	}

	/**
	 * Returns a parser identified from the parser annotation setted on a field
	 * as a custom parser.
	 * 
	 * @param field
	 *            the field
	 * @param parserAnnotation
	 *            the annotation setted on the field.
	 * @param fieldConcreteClass
	 *            the concrete class from which instanciate the field values.
	 * @param <T>
	 *            the type the parser must return
	 * @return the parser
	 * @throws AnnotationProcessorException
	 *             if an error occurs
	 */
	private <T> AbstractParser<T> getCustomParser(Field field, final Parser parserAnnotation,
			Class<T> fieldConcreteClass) throws AnnotationProcessorException {

		AbstractParser<T> parser = null;
		boolean errorOccurs = false;

		TypeVariable[] types = parserAnnotation.value().getTypeParameters();

		// TODO faut-il vraiment tester le type retourné par le parseur?
		if (types.length != 1) {
			LOGGER.warn("{}@Parser : dunno if {} parser deals with {} class", new Object[]{field.getName(),
					parserAnnotation.value().getSimpleName(), fieldConcreteClass.getSimpleName()});
		} else {
			types[0].getClass().asSubclass(fieldConcreteClass);
			try {
				types[0].getClass().asSubclass(fieldConcreteClass);
			} catch (ClassCastException ex) {
				errorOccurs = true;
				LOGGER.error(String.format(
						"%s@Parser : %s field class must be the class (or super class)  the parser deals with",
						field.getName(), fieldConcreteClass.getSimpleName()), ex);
			}
		}

		if (!Util.isConcrete(fieldConcreteClass)) {
			errorOccurs = true;
			LOGGER.error("{}@Parser : {} field class must be a concrete class", field.getName(),
					fieldConcreteClass.getSimpleName());
		}

		// instanciation
		try {

			parser = (AbstractParser<T>) parserAnnotation.value().newInstance();
		} catch (InstantiationException e) {
			errorOccurs = true;
			LOGGER.error(String.format("%s@Parser : %s can not be instanciated", field.getName(), parserAnnotation
					.value().getSimpleName()), e);
		} catch (IllegalAccessException e) {
			errorOccurs = true;
			LOGGER.error(String.format("%s@Parser %s can not be instanciated", field.getName(), parserAnnotation
					.value().getSimpleName()), e);
		}

		// mise en place des propriétés.
		for (Param param : parserAnnotation.params()) {
			String name = param.name();
			String value = param.value();
			Method setter = Util.identifySetter(parserAnnotation.value(), name);

			if (setter == null) {
				LOGGER.error("{}@Parser : no setter found for its {} property", field.getName(), name);
				errorOccurs = true;
			} else {

				Class<?> setterParamClass = setter.getParameterTypes()[0];
				try {
					AbstractParser<?> setterParser = AbstractParser.identifyDefaultParser(setterParamClass);
					if (parser != null) {
						// ON tente de setter la propriété
						try {
							setter.invoke(parser, setterParser.parse(value));
						} catch (IllegalAccessException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Parser : can not setting %s with value %s ",
									field.getName(), name, value), e);
							errorOccurs = true;
						} catch (IllegalArgumentException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Parser : can not setting %s with value %s ",
									field.getName(), name, value), e);
						} catch (InvocationTargetException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Parser : can not setting %s with value %s ",
									field.getName(), name, value), e);
						} catch (ParseException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Parser : can not parse %s to set %s ", field.getName(),
									value, name), e);
						}
					}
				} catch (UsageInvalideException ex) {
					LOGGER.error("{}@Parser : {} property unable to set - Perhaps if the property class has a string "
							+ "argument constructor, it will be better ... ", new Object[]{field.getName(), name});
					errorOccurs = true;
				}
			}
		}

		if (errorOccurs) {
			throw new AnnotationProcessorException();
		}
		return parser;
	}
}
