/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.filter.impl;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.filter.AbstractStringFilter;

/**
 * Filter which extracts a substring by splitting the given string around
 * matches of an regular expression.
 * 
 * @author Olivier Godineau
 * 
 */
public final class SplitFilter extends AbstractStringFilter {
	/**
	 * the reg exp separator.
	 */
	private String separator = null;

	/**
	 * index of the expression to extract.
	 */
	private int index;

	/**
	 * 
	 * @param index
	 *            the n-th expression found by splitting a string around a
	 *            separator.
	 * @param separator
	 *            a regular expression. Must not be <code>null</code>
	 */
	public SplitFilter(int index, String separator) {
		super();
		this.index = index;
		this.separator = separator;
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the expression found at the index after splitting the value
	 * arount the separator. <code>null</code> if the splitted value length is
	 * less or equals than the index.
	 * </p>
	 * <p>
	 * throws {@link UsageInvalideException} if separator is <code>null</code>
	 * </p>
	 */
	@Override
	protected String doFiltre(String value) {
		if (separator == null) {
			throw new UsageInvalideException("SplitFilter.separator must be not null");
		}
		String retour = null;
		if (value != null) {
			String[] values = value.split(separator);
			if (values.length > index) {
				retour = values[index];
			}
		}

		return retour;
	}

	/**
	 * 
	 * @return the separator.
	 */
	public String getSeparator() {
		return separator;
	}

	/**
	 * Sets the separator.
	 * 
	 * @param separator
	 *            the separator. Could be a regexp.
	 */
	public void setSeparator(String separator) {
		this.separator = separator;
	}

	/**
	 * 
	 * @return the index.
	 */
	public int getIndex() {
		return index;
	}

	/**
	 * Sets the index.
	 * 
	 * @param index
	 *            the index.
	 */
	public void setIndex(int index) {
		this.index = index;
	}

}