/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader;

import java.util.Date;
import java.util.Locale;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import olg.csv.bean.formatter.Formatter;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * 
 * Class dedicated to load a formatter {@link Formatter} from an XML element
 * conformed to XML schema FormatterType specification. Each concret class shall
 * have the responsibility to load a concrete Formatter class.
 * 
 * <p>
 * Class based on Chain of responsability pattern and used to chain concret
 * Formatter Loaders.
 * </p>
 * 
 * @author Olivier Godineau
 * 
 */
public abstract class AbstractFormatterLoader {
	/**
	 * Singleton instance to use to load Formatter.
	 */
	private static AbstractFormatterLoader instance = new DateFormatterLoader(new CustomFormatterLoader(null));

	/**
	 * Returns singleton responsible of formatters loading.
	 * 
	 * This is this instance you have to be used to load every type of
	 * formatters from XML element conformed to XML Schema FormatterType
	 * specification.
	 * 
	 * @return the AbstractFormatter loader
	 */
	public static AbstractFormatterLoader getInstance() {
		return instance;
	}

	/**
	 * FormatterLoader successor as describe in Chain of responsability pattern.
	 */
	protected AbstractFormatterLoader successor;

	/**
	 * This Xpath expression allows to identify xml node which describe the
	 * corresponding concret formatter under formatterType node.
	 */
	protected String xPathExpression;

	/**
	 * Returns a concret formatter identified from the given XML element.
	 * 
	 * @param element
	 *            XML node corresponding to correspondant to XML FormatterType
	 *            as described in our XML schema.
	 * @return a concret formatter or <code>null</code> if the given element
	 *         doesn't match the concret formatter type this loader has
	 *         responsability and doesn't match any successor
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 */
	public final Formatter<?> getFormatter(Element element) throws XPathExpressionException, LoadException {
		Formatter<?> retour = null;

		if (element != null) {
			Node node = (Node) Util.evaluerDOM(element, xPathExpression, XPathConstants.NODE);
			if (node != null) { // NOPMD by olivier on 01/02/12 00:43
				retour = getConcreteFormatter((Element) node);
			} else {
				if (successor != null) {
					retour = successor.getFormatter(element);
				}
			}
		}
		return retour;
	}

	/**
	 * 
	 * Returns a concret formatter.
	 * 
	 * @param node
	 *            XML Element corresponding to the XPath expression. From this
	 *            element a concret formatter this loader should return will be
	 *            extracted
	 * 
	 * @return a concret formatter this loader has responsability or
	 *         <code>null</code> if the given node doesn't match the type this
	 *         loader should return
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 * @see #xPathExpression
	 */
	protected abstract Formatter<?> getConcreteFormatter(Element node) throws XPathExpressionException, LoadException;

	/**
	 * 
	 * @param xPathExpression
	 *            XPath expression that allows to identify the concret formatter
	 *            under the XML FormatterType node
	 * @param successor
	 *            following concret loader in the chain of responsability
	 * 
	 */
	protected AbstractFormatterLoader(String xPathExpression, AbstractFormatterLoader successor) {
		super();
		this.successor = successor;
		this.xPathExpression = xPathExpression;
	}

	/**
	 * Class responsible of DateFormatter loading.
	 * 
	 * @see Formatter#getDateFormatter(String, Locale)
	 */
	private static class DateFormatterLoader extends AbstractFormatterLoader {

		/**
		 * 
		 * @param successor
		 *            the next loader in the formatter loaders chain.
		 */
		protected DateFormatterLoader(AbstractFormatterLoader successor) {
			super("date", successor);

		}

		@Override
		protected Formatter<? extends Date> getConcreteFormatter(Element node) throws XPathExpressionException,
				LoadException {
			String loc = node.getAttribute("locale");
			Locale locale = Util.getLocale(loc);
			if (!"".equals(loc) && locale == null) {
				throw new LoadException("Locale based on expression[" + loc + "] is not available ");
			}

			return Formatter.getDateFormatter(node.getAttribute("format"), locale);
		}

	}

	/**
	 * Class responsible of CustomFormatter loading.
	 * 
	 * @see CustomLoader
	 */
	private static class CustomFormatterLoader extends AbstractFormatterLoader {

		/**
		 * 
		 * @param successor
		 *            the next loader in the formatter loaders chain.
		 */
		protected CustomFormatterLoader(AbstractFormatterLoader successor) {
			super("custom", successor);

		}

		@Override
		protected Formatter<?> getConcreteFormatter(Element node) throws XPathExpressionException, LoadException {
			Object object = CustomLoader.getBean(node);
			if (!(object instanceof Formatter)) {
				throw new LoadException(object.getClass() + " must extend " + Formatter.class);
			}

			return (Formatter<?>) object;
		}
	}
}
