/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader.filter;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import olg.csv.bean.filter.AbstractStringFilter;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.loader.Util;
import olg.csv.bean.loader.filter.impl.CustomFilterLoader;
import olg.csv.bean.loader.filter.impl.DecorateFilterLoader;
import olg.csv.bean.loader.filter.impl.LowerFilterLoader;
import olg.csv.bean.loader.filter.impl.ReplaceFilterLoader;
import olg.csv.bean.loader.filter.impl.SplitFilterLoader;
import olg.csv.bean.loader.filter.impl.TrimFilterLoader;
import olg.csv.bean.loader.filter.impl.UpperFilterLoader;
import olg.csv.bean.loader.filter.impl.UpperFirstFilterLoader;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Class dedicated to load a filter {@link AbstractStringFilter} from an XML
 * element conformed to XML schema FilterType specification. Each concret class
 * shall have the responsibility to load a concrete AbstractStringFilter class.
 * 
 * <p>
 * Class based on Chain of responsability pattern and used to chain concret
 * Filter Loaders.
 * </p>
 * 
 * @author Olivier Godineau
 * @see AbstractStringFilter
 * 
 */
public abstract class AbstractFiltreLoader {

	/**
	 * Singleton responsible of filters loading.
	 */
	private static AbstractFiltreLoader instance = new ReplaceFilterLoader(new SplitFilterLoader(
			new DecorateFilterLoader(new UpperFirstFilterLoader(new UpperFilterLoader(new LowerFilterLoader(
					new TrimFilterLoader(new CustomFilterLoader(null))))))));

	/**
	 * Returns singleton responsible of filters loading.
	 * 
	 * This is this instance you have to be used to load every type of filter
	 * from XML element conformed to XML Schema FitlerType specification.
	 * 
	 * @return the abstract filter loader.
	 */
	public static AbstractFiltreLoader getInstance() {
		return instance;

	}

	/**
	 * FiltreLoader successor as describe in Chain of responsability pattern.
	 * 
	 */
	protected AbstractFiltreLoader successor;

	/**
	 * This Xpath expression allows to identify xml node which describe the
	 * corresponding concret filter under Fitler Type node.
	 */
	protected String xPathExpression;

	/**
	 * Returns a concret filter identified from the given XML element.
	 * 
	 * @param element
	 *            XML node corresponding to correspondant to XML FilterType as
	 *            described in our XML schema.
	 * @return a concret filter or <code>null</code> if the given element
	 *         doesn't match concret filter type this loader has responsability
	 *         and doesn't match any successor
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 */
	public final AbstractStringFilter getFilter(Element element) throws XPathExpressionException, LoadException {
		AbstractStringFilter retour = null;
		if (element != null) {
			Node node = (Node) Util.evaluerDOM(element, xPathExpression, XPathConstants.NODE);
			if (node != null) { // NOPMD by olivier on 01/02/12 00:43
				retour = getConcreteFilter((Element) node);
				retour.setFilter(AbstractFiltreLoader.getInstance().getFilter((Element) node));

			} else {
				if (successor != null) {
					retour = successor.getFilter(element);
				}
			}
		}
		return retour;
	}

	/**
	 * 
	 * Returns a concret filter.
	 * 
	 * @param node
	 *            XML Element corresponding to the XPath expression from which
	 *            the concret filter will be loaded
	 * 
	 * @return a concret filter this loader has responsability or
	 *         <code>null</code> if the given node doesn't match the type this
	 *         loader should return
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 * @see #xPathExpression
	 */
	protected abstract AbstractStringFilter getConcreteFilter(Element node) throws XPathExpressionException,
			LoadException;

	/**
	 * 
	 * @param xPathExpression
	 *            XPath expression that allows to identify the concret filter
	 *            under the XML FilterType node
	 * @param successor
	 *            following concret loader in the chain of responsability
	 * 
	 */
	protected AbstractFiltreLoader(String xPathExpression, AbstractFiltreLoader successor) {
		super();
		this.successor = successor;
		this.xPathExpression = xPathExpression;
	}
}
