/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader.parser;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.loader.Util;
import olg.csv.bean.loader.parser.impl.CustomParserLoader;
import olg.csv.bean.loader.parser.impl.DateParserLoader;
import olg.csv.bean.parser.AbstractParser;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Class dedicated to load a parser from an XML element conformed to XML schema
 * ParserType specification. Each concret class shall have the responsibility to
 * load a concrete AbstractLoader class.
 * 
 * <p>
 * Class based on Chain of responsability pattern used to chain concret Filter
 * Loaders.
 * </p>
 * 
 * @author Olivier Godineau
 * @see AbstractParser
 * 
 */
public abstract class AbstractParserLoader {

	/**
	 * Singleton responsible of parsers loading.
	 */
	private static AbstractParserLoader instance = new DateParserLoader(new CustomParserLoader(null));

	/**
	 * Returns Singleton responsible of parsers loading. This is this instance
	 * you have to be used to load every type of parser from XML element
	 * conformed to XML Schema ParserType specification.
	 * 
	 * @return the parser loader instance.
	 */
	public static AbstractParserLoader getInstance() {
		return instance;
	}

	/**
	 * This Xpath expression allows to identify xml node which describe the
	 * corresponding concret parser under parser Type node.
	 */
	protected String xPathExpression;

	/**
	 * ParserLoader successor as describe in Chain of responsability pattern.
	 * Each parserLoader has the responsability of only one parser type.
	 */
	protected AbstractParserLoader successor;

	/**
	 * 
	 * Returns a concret parser identified from the given XML Element.
	 * 
	 * @param clazz
	 *            the class that the parser should return
	 * 
	 * @param element
	 *            XML node corresponding to correspondant to XML ParserType as
	 *            described in our XML schema
	 * @param <T>
	 *            the type the parser treats.
	 * @return a concret parser or <code>null</code> if the given element
	 *         doesn't match concret parser this loader has responsability and
	 *         doesn't match any successor
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 */
	@SuppressWarnings("unchecked")
	public final <T> AbstractParser<T> getParser(Class<T> clazz, Element element) throws XPathExpressionException,
			LoadException {
		AbstractParser<T> retour = null;
		if (element == null) {
			try {
				retour = AbstractParser.identifyDefaultParser(clazz);
			} catch (UsageInvalideException e) {
				throw new LoadException(e);
			}

		} else {
			Node node = (Node) Util.evaluerDOM(element, xPathExpression, XPathConstants.NODE);
			if (node != null) { // NOPMD by olivier on 01/02/12 00:50
				retour = getConcreteParser(clazz, (Element) node);
			} else {
				if (successor != null) {
					retour = successor.getParser(clazz, element);
				}
			}
		}
		return retour;
	}

	/**
	 * 
	 * @param xPathExpression
	 *            the XPath expression that allows to identify the concret
	 *            parser under the XML FilterType node
	 * @param successor
	 *            following concret loader in the chain of responsability
	 */
	protected AbstractParserLoader(String xPathExpression, AbstractParserLoader successor) {
		super();
		this.successor = successor;
		this.xPathExpression = xPathExpression;
	}

	/**
	 * Returns a concret parser this concret loader has the responsablity to
	 * create.
	 * 
	 * @param clazz
	 *            the class that the parser should return
	 * @param node
	 *            XML Element corresponding to the XPath expression from which
	 *            load corresponding parser
	 * @param <T>
	 *            the type the parser treats.
	 * @return the concret parser. <code>null</code> if the given node doesn't
	 *         match the type this loader should return
	 * 
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             if Error occurs during loading
	 * @see #xPathExpression
	 */
	protected abstract <T> AbstractParser<T> getConcreteParser(Class<T> clazz, Element node)
			throws XPathExpressionException, LoadException;

}
