/*
 * Copyright 2013 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.annotations.processor;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import olg.csv.base.Cell;
import olg.csv.bean.IPropertyProcessor;
import olg.csv.bean.Util;
import olg.csv.bean.annotations.Column;
import olg.csv.bean.formatter.Formatter;
import olg.csv.bean.getter.AbstractGetter;
import olg.csv.bean.impl.AbstractPropertyProcessor;
import olg.csv.bean.impl.CellProcessor;
import olg.csv.bean.impl.PropertyFormatter;
import olg.csv.bean.parser.AbstractParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class is responsible for processing the @Column annotation.
 * 
 * @author Olivier Godineau
 * 
 * @param <B>
 *            the bean on which research its fields annoted with column
 *            annotation
 */
public final class ColumnProcessor<B> {

	/**
	 * Class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(ColumnProcessor.class);
	/**
	 * Default value assigned as defaultValue column attribute. Used to indicate
	 * the default value is <code>null</code>.
	 */
	private static final String DEFAULT_DEFAULT_VALUE = "null";
	/**
	 * "null" as default value. Used to indicate the default value is the string
	 * "null".
	 */
	private static final String NULL_DEFAULT_VALUE = "\"null\"";

	/**
	 * the class of the bean on which research its fields annoted with column
	 * annotation.
	 */
	private Class<B> clazz;

	/**
	 * Constructor.
	 * 
	 * @param clazz
	 *            the class of the bean.
	 */
	public ColumnProcessor(Class<B> clazz) {
		this.clazz = clazz;
	}

	/**
	 * Complete a map with the cellprocessor built from the column annotation
	 * setted on a field.
	 * 
	 * @param field
	 *            the field (of the beanClass) from which build the cell
	 *            processor.
	 * @param map
	 *            the map to complete with the CellProcessor built from column
	 *            annotation setted on the field. the keys are the cell num
	 *            target of the cellProcessor.
	 * @return true if no error happens during this operation.
	 */
	public boolean getWritingProcessor(Field field, Map<Integer, List<CellProcessor<B>>> map) {
		Column column = field.getAnnotation(Column.class);

		boolean noError = true;

		if (column != null) {
			String rang = column.value();
			int num = 0;
			String name = column.name();

			Method getter = Util.identifyGetter(field.getDeclaringClass(), field.getName());
			Class<?> fieldConcreteClass = field.getType();
			if (!java.lang.Object.class.equals(column.concreteClass())) {
				fieldConcreteClass = column.concreteClass();
				noError &= chekcAsSubClass(field, column);
			}
			noError &= checkConcreteClass(field, fieldConcreteClass);
			try {
				num = Cell.fromSheetCellNumber(rang);
				if ("".equals(name)) {
					name = Cell.defaultCellName(rang);
				}

			} catch (IllegalArgumentException ex) {
				LOGGER.error("{}@Column : {}", field.getName(), ex.getMessage());
				noError = false;
			}

			if (getter == null) {
				LOGGER.error("{}@Column : No public getter found", field.getName());
				noError = false;
			}

			Formatter<?> formatter;
			try {
				formatter = FormatterProcessor.getInstance().getFormatter(field, fieldConcreteClass);
				if (noError) {

					if (map.get(num) == null) {
						map.put(num, new ArrayList<CellProcessor<B>>());
					}

					map.get(num).add(
							new CellProcessor<B>(rang, name, PropertyFormatter.getPropertyReader(getter,
									field.getName(), formatter, null)));
				}
			} catch (AnnotationProcessorException e) {
				noError = false;
			}

		}
		return noError;

	}

	/**
	 * Complete a list with the propertyProcessor built from the column
	 * annotation setted on a field.
	 * 
	 * @param translate
	 *            column index to add to the column num found on the field.
	 *            Useful to define the index of a field herited by an embedded
	 *            attribute.
	 * @param field
	 *            the field (of the beanClass) from which read the Column
	 *            annotation
	 * @param propertyProcessors
	 *            the list to complete
	 * @param map
	 *            the map to complete. this map shows for each cell num the
	 *            count of the property processors setted on.
	 * @return true if no errors occurs.
	 */
	public boolean getReadingProcessor(int translate, Field field, List<IPropertyProcessor<B>> propertyProcessors,
			Map<Integer, Integer> map) {
		Column column = field.getAnnotation(Column.class);
		boolean noError = true;

		if (column != null) {
			String rang = column.value();
			int num = 0;

			Class<?> fieldConcreteClass = field.getType();
			if (!java.lang.Object.class.equals(column.concreteClass())) {
				fieldConcreteClass = column.concreteClass();
				noError &= chekcAsSubClass(field, column);
			}
			noError &= checkConcreteClass(field, fieldConcreteClass);

			AbstractGetter getter = null;
			IPropertyProcessor<B> propertyProcessor = null;
			AbstractParser<?> parser = null;

			try {
				num = Cell.fromSheetCellNumber(rang) + translate;
				if (map.get(num) == null) {
					map.put(num, 0);
				}
				map.put(num, map.get(num) + 1);

				getter = AbstractGetter.getDefault(String.valueOf(num), getDefaultValue(column));

			} catch (IllegalArgumentException ex) {
				LOGGER.error("{}@Column : {}", field.getName(), ex.getMessage());
				noError = false;
			}

			try {
				parser = ParserProcessor.getInstance().getParser(field, fieldConcreteClass);
			} catch (AnnotationProcessorException ex) {
				noError = false;
			}
			Method setter = Util.identifySetter(field.getDeclaringClass(), field.getName());

			if (setter == null) {
				LOGGER.error("{}@Column : {} has no public setter for this field", field.getName(), field
						.getDeclaringClass().getSimpleName());
				noError = false;
			}
			if (parser != null && getter != null && setter != null) {
				try {
					propertyProcessor = AbstractPropertyProcessor.getSimplePropertyTransformer(getter, parser,
							field.getName(), this.clazz);
					propertyProcessors.add(propertyProcessor);
				} catch (IllegalArgumentException ex) {
					LOGGER.error("{}@Column : {}", field.getName(), ex.getMessage());
					noError = false;
				}
			}

		}

		return noError;
	}

	/**
	 * check if the concrete class declared on a column is a subclass of a
	 * field.
	 * 
	 * @param field
	 *            the field
	 * @param column
	 *            the column annotation setted on the field
	 * @return true if the concrete class is a subclass of the field.
	 */
	private boolean chekcAsSubClass(Field field, Column column) {
		boolean noError = true;
		if (!Util.asSubClass(field.getType(), column.concreteClass())) {
			noError = false;
			LOGGER.error("{}@Column : {} concreteClass attribute is not a subclass", field.getName(),
					column.concreteClass());
		}
		return noError;
	}

	/**
	 * Check if a class is concrete.
	 * 
	 * @param field
	 *            the field on which the processor works.
	 * @param fieldConcreteClass
	 *            concrete class to use to instanciate the field.
	 * @return true if the class is concrete.
	 */
	private boolean checkConcreteClass(Field field, Class<?> fieldConcreteClass) {
		if (!Util.isConcrete(fieldConcreteClass)) {
			LOGGER.error("{}@Column : {}  field class is not a concrete class", field.getName(),
					fieldConcreteClass.getSimpleName());
			return false;
		}
		return true;
	}

	/**
	 * Return the default value defined on a column annotation.
	 * 
	 * @param column
	 *            the column
	 * @return the default value as String.
	 */
	private String getDefaultValue(Column column) {
		String value = column.defaultValue();
		if (DEFAULT_DEFAULT_VALUE.equals(value)) {
			value = null;
		}
		if (NULL_DEFAULT_VALUE.equals(value)) {
			value = DEFAULT_DEFAULT_VALUE;
		}
		return value;
	}

}