/*
 * Copyright 2013 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.annotations.processor;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.TypeVariable;
import java.util.Date;
import java.util.Locale;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.Util;
import olg.csv.bean.annotations.DateProperty;
import olg.csv.bean.annotations.Param;
import olg.csv.bean.formatter.Formatter;
import olg.csv.bean.parser.AbstractParser;
import olg.csv.bean.parser.ParseException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This class is responsible for processing the @Formatter annotation.
 * 
 * @author Olivier Godineau
 * 
 */
public final class FormatterProcessor {
	/**
	 * The class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(FormatterProcessor.class);

	/**
	 * Singleton instance.
	 */
	private static FormatterProcessor instance = new FormatterProcessor();

	/**
	 * Private constructor to ensure the existence of singleton.
	 */
	private FormatterProcessor() {

	}

	/**
	 * The singleton instance.
	 * 
	 * @return the singleton instance
	 */
	public static FormatterProcessor getInstance() {
		return instance;
	}

	/**
	 * Return the formatter to use on a field. If no formatter annotation is
	 * setted on the field, a default formatter is returned (use the toString
	 * method of the concrete class)
	 * 
	 * @param field
	 *            the field to format
	 * @param fieldConcreteClass
	 *            the concrete class
	 * @param <T>
	 *            the type the formatter deals with.
	 * @return the formatter
	 * @throws AnnotationProcessorException
	 *             if an error occurs
	 */
	public <T> Formatter<T> getFormatter(Field field, Class<T> fieldConcreteClass) throws AnnotationProcessorException {
		Formatter<T> formatter = null;

		olg.csv.bean.annotations.Formatter formatterAnnotation = field
				.getAnnotation(olg.csv.bean.annotations.Formatter.class);
		if (formatterAnnotation != null) {
			Class<? extends Formatter<?>> clazz = formatterAnnotation.value();
			if (clazz.equals(olg.csv.bean.formatter.DateFormatter.class)) {
				formatter = (Formatter<T>) getDateFormatter(field, formatterAnnotation, fieldConcreteClass);
			} else {

				formatter = getCustomFormatter(field, formatterAnnotation, fieldConcreteClass);
			}
		} else {
			// formatter par défaut;
			formatter = new Formatter<T>();
		}

		return formatter;

	}

	/**
	 * Returns a date formatter identified from the formatter annotation setted
	 * on a field.
	 * 
	 * @param field
	 *            the field
	 * @param formatterAnnotation
	 *            the annotation setted on the field.
	 * @param fieldConcreteClass
	 *            the concrete class the formatter deals with.
	 * @param <T>
	 *            the type the formatte deals with.
	 * @return the Date formatter
	 * @throws AnnotationProcessorException
	 *             if an error occurs
	 */
	private <T> Formatter<Date> getDateFormatter(Field field, olg.csv.bean.annotations.Formatter formatterAnnotation,
			Class<T> fieldConcreteClass) throws AnnotationProcessorException {
		Formatter<Date> formatter = null;
		String format = null;
		Locale locale = null;

		boolean errorOccurs = false;

		try {
			fieldConcreteClass.asSubclass(Date.class);
		} catch (ClassCastException ex) {
			errorOccurs = true;
			LOGGER.error("{}@Formatter : {} field class must extend Date class to instanciate a Date Formatter",
					field.getName(), fieldConcreteClass.getSimpleName());
		}

		if (fieldConcreteClass.isInterface()) {
			errorOccurs = true;
			LOGGER.error("{}@Formatter : {} field class must be a concret class", field.getName(),
					fieldConcreteClass.getSimpleName());
		}

		for (Param param : formatterAnnotation.params()) {
			if (param.name().equals(DateProperty.FORMAT)) {
				format = param.value();
			} else {
				if (param.name().equals(DateProperty.LOCALE)) {
					locale = olg.csv.bean.loader.Util.getLocale(param.value());
					if (locale == null) {
						LOGGER.warn("{}@Formatter : {} Locale property is not an available Locale, it will be ignored",
								field.getName(), param.value());
					}
				} else {
					LOGGER.warn("{}@Formatter : {} is not a DateFormatter legal property, it will be ignored ",
							field.getName(), param.name());
				}
			}
		}

		if (format == null) { // Format is required
			errorOccurs = true;
			LOGGER.error("{}@Formatter : DateFormatter {} property is required", field.getName(), DateProperty.FORMAT);
		} else {
			try {
				formatter = Formatter.getDateFormatter(format, locale);

			} catch (UsageInvalideException ex) {
				errorOccurs = true;
				LOGGER.error(String.format("%s@Formatter : DateParser cannot be instanciated", field.getName()), ex);
			}
		}

		if (errorOccurs) {
			throw new AnnotationProcessorException();
		}
		return formatter;
	}

	/**
	 * Returns a formatter identified from the formatter annotation setted on a
	 * field as a custom formatter.
	 * 
	 * @param field
	 *            the field.
	 * @param formatterAnnotation
	 *            the annotatin setted on the field.
	 * @param fieldConcreteClass
	 *            the concrete class the formatter deals with.
	 * @param <T>
	 *            the type the formatter deals with.
	 * @return the custom formatter
	 * @throws AnnotationProcessorException
	 *             if an error occurs
	 */
	private <T> Formatter<T> getCustomFormatter(Field field, olg.csv.bean.annotations.Formatter formatterAnnotation,
			Class<T> fieldConcreteClass) throws AnnotationProcessorException {

		Formatter<T> formatter = null;
		boolean errorOccurs = false;

		TypeVariable[] types = formatterAnnotation.value().getTypeParameters();
		if (types.length != 1) {
			LOGGER.warn("{}@Formatter : dunno if {} formatter deals with {} class", new Object[]{field.getName(),
					formatterAnnotation.value().getSimpleName(), fieldConcreteClass.getSimpleName()});
		} else {

			try {
				types[0].getClass().asSubclass(fieldConcreteClass);
			} catch (ClassCastException ex) {
				errorOccurs = true;
				LOGGER.error(String.format(
						"%s@Formatter : %s field class must be the class (or super class)  the formatter deals with",
						field.getName(), fieldConcreteClass.getSimpleName()), ex);
			}
		}

		if (!Util.isConcrete(fieldConcreteClass)) {
			errorOccurs = true;
			LOGGER.error("{}@Formatter : {} field class must be a concret class", field.getName(),
					fieldConcreteClass.getSimpleName());
		}

		// instanciation
		try {

			formatter = (Formatter<T>) formatterAnnotation.value().newInstance();
		} catch (InstantiationException e) {
			errorOccurs = true;
			LOGGER.error(String.format("%s@Formatter : %s can not be instanciated", field.getName(),
					formatterAnnotation.value().getSimpleName()), e);
		} catch (IllegalAccessException e) {
			errorOccurs = true;
			LOGGER.error(String.format("%s@Formatter : %s can not be instanciated", field.getName(),
					formatterAnnotation.value().getSimpleName()), e);
		}

		// mise en place des propriétés.
		for (Param param : formatterAnnotation.params()) {
			String name = param.name();
			String value = param.value();
			Method setter = Util.identifySetter(formatterAnnotation.value(), name);

			if (setter == null) {
				LOGGER.error("{}@Formatter : no setter found for its {} property", field.getName(), name);
				errorOccurs = true;
			} else {

				Class<?> setterParamClass = setter.getParameterTypes()[0];
				try {
					AbstractParser<?> setterParser = AbstractParser.identifyDefaultParser(setterParamClass);
					if (formatter != null) {
						// ON tente de setter la propriété
						try {
							setter.invoke(formatter, setterParser.parse(value));
						} catch (IllegalAccessException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Formatter : can not set %s with value %s ", field.getName(),
									name, value), e);
							errorOccurs = true;
						} catch (IllegalArgumentException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Formatter : can not set %s with value %s ", field.getName(),
									name, value), e);
						} catch (InvocationTargetException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Formatter : can not set %s with value %s ", field.getName(),
									name, value), e);
						} catch (ParseException e) {
							errorOccurs = true;
							LOGGER.error(String.format("%s@Formatter : can not parse %s to set %s ", field.getName(),
									value, name), e);
						}
					}
				} catch (UsageInvalideException ex) {
					LOGGER.error(
							"{}@Formatter : {} property unable to set - Perhaps if the property class has a string "
									+ "argument constructor, it will be better ... ", new Object[]{field.getName(),
									name});
					errorOccurs = true;
				}
			}
		}

		if (errorOccurs) {
			throw new AnnotationProcessorException();
		}
		return formatter;
	}
}
