/*
 * Copyright 2013 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.annotation.processor;

import java.text.SimpleDateFormat;
import java.util.ArrayList;

import olg.csv.base.Cell;
import olg.csv.base.Row;
import olg.csv.base.csv.CSVSettings;
import olg.csv.base.ods.ODSSettings;
import olg.csv.bean.IBeanProcessor;
import olg.csv.bean.IRowProcessor;
import olg.csv.bean.annotation.test.model.TestBean;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorBean;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorBean.TestRowBeanProcessorChildBean;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorFieldsOnSameColumn;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorNotAnnotated;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorParentSettingsBean;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorSettingsBean;
import olg.csv.bean.annotation.test.model.TestRowBeanProcessorSettingsBean2;
import olg.csv.bean.annotations.processor.AnnotationProcessorException;
import olg.csv.bean.annotations.processor.RowBeanProcessor;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.JUnit4;

@RunWith(JUnit4.class)
public final class RowBeanProcessorTest {

	/**
	 * hasRowBeanAnnotation method test.
	 */
	@Test
	public void hasRowBeanAnnotationTest() {
		RowBeanProcessor<TestRowBeanProcessorSettingsBean> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean>(
				TestRowBeanProcessorSettingsBean.class);

		Assert.assertTrue(processor.hasRowBeanAnnotation());
		RowBeanProcessor<TestRowBeanProcessorSettingsBean2> processor2 = new RowBeanProcessor<TestRowBeanProcessorSettingsBean2>(
				TestRowBeanProcessorSettingsBean2.class);
		Assert.assertTrue(processor2.hasRowBeanAnnotation());

		RowBeanProcessor<TestRowBeanProcessorParentSettingsBean> processorParent = new RowBeanProcessor<TestRowBeanProcessorParentSettingsBean>(
				TestRowBeanProcessorParentSettingsBean.class);
		Assert.assertTrue(processorParent.hasRowBeanAnnotation());

		RowBeanProcessor<TestBean> processor3 = new RowBeanProcessor<TestBean>(TestBean.class);
		Assert.assertFalse(processor3.hasRowBeanAnnotation());

	}

	/**
	 * Test inherited RowBean options from parent class.
	 * 
	 * @throws AnnotationProcessorException
	 */
	@Test
	public void getReadingCSVSettingstest() throws AnnotationProcessorException {

		RowBeanProcessor<TestRowBeanProcessorSettingsBean> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean>(
				TestRowBeanProcessorSettingsBean.class);
		CSVSettings settings = processor.getReadingCSVSettings();
		Assert.assertNotNull("TestBean must have CSVSettings  inherited from TestParentBean", settings);

		Assert.assertEquals("TestBeanParent has withHeaders csvsettings", true, settings.isWithHeaders());
		Assert.assertEquals(';', settings.getSeparator());
		Assert.assertEquals('\'', settings.getDelimiter());
		Assert.assertEquals("ISO-8859-1", settings.getCharsetName());

		// LineSeparator is ignored on reading
		Assert.assertNotEquals("x", settings.getLineSeparator());
		Assert.assertEquals(CSVSettings.DEFAULT_LINESEPARATOR, settings.getLineSeparator());

	}

	/**
	 * Test when child class overrides parent RowBean Annotation.
	 * 
	 * @throws AnnotationProcessorException
	 */
	@Test
	public void getReadingCSVSettingstest2() throws AnnotationProcessorException {

		RowBeanProcessor<TestRowBeanProcessorSettingsBean2> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean2>(
				TestRowBeanProcessorSettingsBean2.class);
		CSVSettings settings = processor.getReadingCSVSettings();
		Assert.assertNotNull("TestRowBeanProcessorSettingsBean2 must have CSVSettings", settings);

		Assert.assertEquals("TestRowBeanProcessorSettingsBean2 has withHeaders csvsettings", true,
				settings.isWithHeaders());
		Assert.assertEquals('x', settings.getSeparator());
		Assert.assertEquals('y', settings.getDelimiter());
		Assert.assertEquals("ANSI", settings.getCharsetName());

		// LineSeparator is ignored on reading
		Assert.assertNotEquals("x", settings.getLineSeparator());
		Assert.assertEquals(CSVSettings.DEFAULT_LINESEPARATOR, settings.getLineSeparator());

	}
	/**
	 * Test when child class overrides parent RowBean Annotation.
	 * 
	 * @throws AnnotationProcessorException
	 */
	@Test
	public void getWritingCSVSettingstest2() throws AnnotationProcessorException {

		RowBeanProcessor<TestRowBeanProcessorSettingsBean2> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean2>(
				TestRowBeanProcessorSettingsBean2.class);
		CSVSettings settings = processor.getWritingCSVSettings();
		Assert.assertNotNull("TestBean must have default CSVSettings ", settings);

		Assert.assertEquals("TestBeanPare has default withHeaders csvsettings", false, settings.isWithHeaders());

		Assert.assertEquals(CSVSettings.DEFAULT_SEPARATOR, settings.getSeparator());
		Assert.assertEquals(CSVSettings.DEFAULT_DELIMITER, settings.getDelimiter());
		Assert.assertEquals(CSVSettings.DEFAULT_CHARSETNAME, settings.getCharsetName());
		Assert.assertEquals(CSVSettings.DEFAULT_LINESEPARATOR, settings.getLineSeparator());

	}

	@Test
	public void getWritingCSVSettingstest() throws AnnotationProcessorException {

		RowBeanProcessor<TestRowBeanProcessorSettingsBean> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean>(
				TestRowBeanProcessorSettingsBean.class);
		CSVSettings settings = processor.getWritingCSVSettings();
		Assert.assertNotNull("TestBean must have CSVSettings  inherited from TestParentBean", settings);
		// unexpected param
		Assert.assertEquals("TestBeanParent has withHeaders csvsettings", true, settings.isWithHeaders());

		Assert.assertEquals(';', settings.getSeparator());
		Assert.assertEquals('\'', settings.getDelimiter());
		Assert.assertEquals("ISO-8859-1", settings.getCharsetName());
		Assert.assertEquals("\r\n", settings.getLineSeparator());
		Assert.assertEquals(CSVSettings.DEFAULT_SKIPEMPTYCELL, settings.isSkipEmptyCell());
		Assert.assertEquals(CSVSettings.DEFAULT_FORCEDELIMITER, settings.isForceDelimiter());

	}

	@Test
	public void getReadingODSSettingstest() throws AnnotationProcessorException {

		RowBeanProcessor<TestRowBeanProcessorSettingsBean> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean>(
				TestRowBeanProcessorSettingsBean.class);
		ODSSettings settings = processor.getReadingODSSettings();
		Assert.assertNotNull("TestBean must have ODSSettings  inherited from TestParentBean", settings);
		Assert.assertEquals(false, settings.isWithHeaders());
		Assert.assertEquals(10, settings.getSheetNum());
		Assert.assertEquals("Characters", settings.getSheetName());
		Assert.assertEquals(Integer.valueOf(5), settings.getBeginAtRow());
		Assert.assertEquals(Integer.valueOf(500), settings.getEndAtRow());
		Assert.assertEquals("C", settings.getBeginAtColumn());
		Assert.assertEquals("E", settings.getEndAtColumn());

	}

	@Test
	public void getWritingODSSettingstest() throws AnnotationProcessorException {

		RowBeanProcessor<TestRowBeanProcessorSettingsBean> processor = new RowBeanProcessor<TestRowBeanProcessorSettingsBean>(
				TestRowBeanProcessorSettingsBean.class);
		ODSSettings settings = processor.getWritingODSSettings();
		Assert.assertNotNull("TestBean must have ODSSettings  inherited from TestParentBean", settings);

		// unexpected param
		Assert.assertEquals(0, settings.getSheetNum());
		Assert.assertEquals("Characters", settings.getSheetName());
		Assert.assertEquals(Integer.valueOf(5), settings.getBeginAtRow());
		// unexpected param
		Assert.assertEquals(null, settings.getEndAtRow());
		Assert.assertEquals("C", settings.getBeginAtColumn());
		// unexpected param
		Assert.assertEquals(null, settings.getEndAtColumn());
	}

	@Test
	public void getRowProcessorTest() {
		RowBeanProcessor<TestRowBeanProcessorChildBean> annotationProcessor = new RowBeanProcessor<TestRowBeanProcessorChildBean>(
				TestRowBeanProcessorChildBean.class);
		try {
			IRowProcessor<TestRowBeanProcessorChildBean> processor = annotationProcessor.getRowProcessor();
			TestRowBeanProcessorChildBean bean = TestRowBeanProcessorBean.getSample();
			Row row = processor.getHeaders();
			Assert.assertEquals("Bool", row.getCell(0).getValue());
			Assert.assertEquals("Date", row.getCell(1).getValue());
			Assert.assertEquals("String", row.getCell(2).getValue());
			Assert.assertEquals("Integer", row.getCell(3).getValue());
			Assert.assertEquals("Childhood", row.getCell(4).getValue());
			row = processor.transform(bean);
			Cell cell = row.getCell(0);
			Assert.assertEquals(bean.getEmbeddedBean().getField1().toString(), cell.getValue());
			cell = row.getCell(1);
			Assert.assertEquals(new SimpleDateFormat("dd/MM/yyyy").format(bean.getEmbeddedBean().getField2()),
					cell.getValue());
			cell = row.getCell(2);
			Assert.assertEquals(bean.getField1(), cell.getValue());
			cell = row.getCell(3);
			Assert.assertEquals(bean.getField2().toString(), cell.getValue());
			cell = row.getCell(4);
			Assert.assertEquals(bean.getChildhood(), cell.getValue());

		} catch (AnnotationProcessorException e) {
			Assert.fail("getRowProcessor fails");

		}
	}
	@Test
	public void getBeanProcessorTest() {
		RowBeanProcessor<TestRowBeanProcessorChildBean> annotationProcessor = new RowBeanProcessor<TestRowBeanProcessorChildBean>(
				TestRowBeanProcessorChildBean.class);

		try {
			IBeanProcessor<TestRowBeanProcessorChildBean> processor = annotationProcessor.getBeanProcessor();
			ArrayList<Cell> cells = new ArrayList<Cell>();
			TestRowBeanProcessorChildBean sample = TestRowBeanProcessorBean.getSample();
			cells.add(new Cell(0, sample.getEmbeddedBean().getField1().toString()));
			cells.add(new Cell(1, new SimpleDateFormat("dd/MM/yyyy").format(sample.getEmbeddedBean().getField2())));
			cells.add(new Cell(2, sample.getField1()));
			cells.add(new Cell(3, sample.getField2().toString()));
			cells.add(new Cell(4, sample.getChildhood()));
			Row row = new Row(1, cells, 5);

			TestRowBeanProcessorChildBean bean = processor.transform(row);
			Assert.assertEquals(sample, bean);

		} catch (AnnotationProcessorException e) {
			Assert.fail("getRowProcessor fails");

		}
	}
	@Test
	public void getRowProcessorKOTest() {
		RowBeanProcessor<TestRowBeanProcessorNotAnnotated> processor = new RowBeanProcessor<TestRowBeanProcessorNotAnnotated>(
				TestRowBeanProcessorNotAnnotated.class);
		try {
			processor.getRowProcessor();
			Assert.fail("processor must fail with a class without RowBean annotation");
		} catch (AnnotationProcessorException e) {

		} catch (Exception ex) {
			Assert.fail("On Error processor must throw an AnnotationProcessorException");
		}

		RowBeanProcessor<TestRowBeanProcessorFieldsOnSameColumn> processor2 = new RowBeanProcessor<TestRowBeanProcessorFieldsOnSameColumn>(
				TestRowBeanProcessorFieldsOnSameColumn.class);

		try {
			processor2.getRowProcessor();
			Assert.fail("processor must fail with a class without RowBean annotation");
		} catch (AnnotationProcessorException e) {

		} catch (Exception ex) {
			Assert.fail("On Error processor must throw an AnnotationProcessorException");
		}

	}

	@Test
	public void getBeanProcessorKOTest() {
		RowBeanProcessor<TestRowBeanProcessorNotAnnotated> processor = new RowBeanProcessor<TestRowBeanProcessorNotAnnotated>(
				TestRowBeanProcessorNotAnnotated.class);
		try {
			processor.getBeanProcessor();
			Assert.fail("processor must fail with a class without RowBean annotation");
		} catch (AnnotationProcessorException e) {

		} catch (Exception ex) {
			Assert.fail("On Error processor must throw an AnnotationProcessorException");
		}

		RowBeanProcessor<TestRowBeanProcessorFieldsOnSameColumn> processor2 = new RowBeanProcessor<TestRowBeanProcessorFieldsOnSameColumn>(
				TestRowBeanProcessorFieldsOnSameColumn.class);
		try {
			processor2.getBeanProcessor();
			Assert.fail("processor must fail with a class without RowBean annotation");
		} catch (AnnotationProcessorException e) {

		} catch (Exception ex) {
			Assert.fail("On Error processor must throw an AnnotationProcessorException");
		}
	}

}
