/*
 * Copyright 2013 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base;

/**
 * Settings Class for spreadsheet document reader and writer class.
 * 
 * @param <T>
 *            the concret settings type.
 * 
 * 
 */
public abstract class AbstractSheetSettings<T extends AbstractSheetSettings<T>> {
	/**
	 * Default with headers value.
	 */
	public static final boolean DEFAULT_WITHHEADERS = true;
	/**
	 * Default sheet num on which read/write.
	 */
	public static final int DEFAULT_SHEETNUM = 0;
	/**
	 * sheet num.
	 */
	protected int sheetNum = DEFAULT_SHEETNUM;
	/**
	 * 
	 */
	protected boolean withHeaders = DEFAULT_WITHHEADERS;
	/**
	 * the row from which begin the reading/writing. first row is 1.
	 */
	protected Integer beginAtRow = 1;
	/**
	 * The row to which ends the reading/writing. Could be <code>null</code>.
	 */
	protected Integer endAtRow = null;
	/**
	 * the index column from which begin row reading/writing.
	 */
	protected String beginAtColumn = null;
	/**
	 * The index column to which end the row reading/writing.
	 */
	protected String endAtColumn = null;
	/**
	 * The sheet name on which read.
	 */
	protected String sheetName = null;

	/**
	 * Purpose default settings.
	 * <ul>
	 * <li>sheetNum = 0 : take the first sheet of the document</li>
	 * <li>withHeaders = true : skip first read line</li>
	 * <li>beginAtRow = 1 : begin reading or writing at the first line</li>
	 * <li>endAtRow = null : the reading ends at the end of the document</li>
	 * <li>beginAtColumn = null : the reading or writing begins at the first
	 * column on each line</li>
	 * <li>endAtColumn = null: the reading</li>
	 * </ul>
	 */
	public AbstractSheetSettings() {
		super();
	}

	/**
	 * 
	 * @param sheetNum
	 *            if sheetName parameter is <code>null</code>,try to load sheet
	 *            with this index in the sheet list.
	 * @param withHeaders
	 *            Only for Reading. If true skip the first read line, considered
	 *            as headers line.
	 * @param beginAtRow
	 *            specify which line of the file, starts the reading or writing,
	 *            If <code>null</code>, start at the first line
	 * @param endAtRow
	 *            specify which line of the file ends the reading. If
	 *            <code>null</code>, end at the last line. Only on reading
	 *            process
	 * @param beginAtColumn
	 *            specify which column of each line starts the reading or
	 *            writing. Sheet cell number format('A', 'B', 'C',..., 'AA',
	 *            ...) or integer Format.May be <code>null</code>, in this case
	 *            start at the first column of each line.
	 * @param endAtColumn
	 *            Specify which column of each line end reading. Sheet cell
	 *            number format('A', 'B', 'C',..., 'AA', ...) or integer Format.
	 *            May be <code>null</code>
	 * @param sheetName
	 *            if specified, try to load sheet with this name (sheetNum
	 *            parameter is ignored then redefined)
	 */
	public AbstractSheetSettings(int sheetNum, boolean withHeaders, Integer beginAtRow, Integer endAtRow,
			String beginAtColumn, String endAtColumn, String sheetName) {
		super();
		this.sheetNum = sheetNum;
		this.withHeaders = withHeaders;
		this.beginAtRow = beginAtRow;
		this.endAtRow = endAtRow;
		this.beginAtColumn = beginAtColumn;
		this.endAtColumn = endAtColumn;
		this.sheetName = sheetName;
	}

	/**
	 * 
	 * @return sheet index in the sheets list of the document
	 */
	public int getSheetNum() {
		return sheetNum;
	}
	/**
	 * Sets the sheet index in the sheets list of the document.
	 * 
	 * @param sheetNum
	 *            the sheet index in the document.
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setSheetNum(int sheetNum) {
		this.sheetNum = sheetNum;
		return (T) this;
	}

	/**
	 * indicate if the first row must be skipped on reading or if an headers
	 * line must be written on writing.
	 * 
	 * @return with headers or not.
	 */
	public boolean isWithHeaders() {
		return withHeaders;
	}

	/**
	 * Allows to skip headers line on reading or allows to write headers on
	 * writing.
	 * 
	 * @param withHeaders
	 *            indicate if the document has an header.
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setWithHeaders(boolean withHeaders) {
		this.withHeaders = withHeaders;
		return (T) this;
	}

	/**
	 * 
	 * @return the row number where to begin reading or writing.
	 */
	public Integer getBeginAtRow() {
		return beginAtRow;
	}
	/**
	 * Sets the row number where to begin reading or writing.
	 * 
	 * @param beginAtRow
	 *            May be <code>null</code>(the reading or writing begins at the
	 *            first line of the document)
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setBeginAtRow(Integer beginAtRow) {
		this.beginAtRow = beginAtRow;
		return (T) this;
	}

	/**
	 * Returns the last row to read if defined. Only for reading.
	 * 
	 * @return the last row to read.
	 */
	public Integer getEndAtRow() {
		return endAtRow;
	}

	/**
	 * Set at which line end reading.Only for reading.
	 * 
	 * @param endAtRow
	 *            May be <code>null</code>(the reading ends at the last line of
	 *            the document)
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setEndAtRow(Integer endAtRow) {
		this.endAtRow = endAtRow;
		return (T) this;
	}

	/**
	 * 
	 * @return the num of the first column to read/write.
	 */
	public String getBeginAtColumn() {
		return beginAtColumn;
	}

	/**
	 * Set from which column begin to read/write each line.
	 * 
	 * @param beginAtColumn
	 *            May be <code>null</code>(each read or written line start at
	 *            the first column)
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setBeginAtColumn(String beginAtColumn) {
		this.beginAtColumn = beginAtColumn;
		return (T) this;
	}

	/**
	 * Returns the num of the last column to read if defined. Only for reading.
	 * 
	 * @return the num of the last column to read.
	 */
	public String getEndAtColumn() {
		return endAtColumn;
	}

	/**
	 * Set to which column end reading each line.Only for reading.
	 * 
	 * @param endAtColumn
	 *            May be <code>null</code>
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setEndAtColumn(String endAtColumn) {
		this.endAtColumn = endAtColumn;
		return (T) this;
	}

	/**
	 * Returns the sheet name.
	 * 
	 * @return the sheet name.
	 */
	public String getSheetName() {
		return sheetName;
	}

	/**
	 * The desire sheet name.
	 * 
	 * @param sheetName
	 *            the name of the sheet in the spreadsheet document.
	 * @return the current instance
	 */
	@SuppressWarnings("unchecked")
	public T setSheetName(String sheetName) {
		this.sheetName = sheetName;
		return (T) this;
	}

}
