/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base;

import java.io.IOException;
import java.util.NoSuchElementException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * A "Proxy" reader which Provides a mecanism to skip empty row.
 * 
 * @author Olivier Godineau
 * 
 */
public class IgnoreNullReader implements IReader {
	/**
	 * The class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(IgnoreNullReader.class);
	/**
	 * the original reader.
	 */
	private final IReader reader;

	/**
	 * the last row retrieved from the original reader.
	 */
	private Row currentRow = null;
	/**
	 * 
	 */
	private boolean next;
	/**
	 * because next() method even if returns the next element, loads the
	 * following row. During loading the following row a readException could
	 * occur. So we need to keep this exception to throw it on the following
	 * call of next() method.
	 */
	private ReaderException exception = null;

	/**
	 * 
	 * @param reader
	 *            a concret reader
	 */
	public IgnoreNullReader(IReader reader) {
		super();
		if (reader == null) {
			throw new IllegalArgumentException(IgnoreNullReader.class.getSimpleName()
					+ " Constructor IReader argument must not be null");
		}
		this.reader = reader;
		this.currentRow = setNext();
	}
	/**
	 * @return true if exists a next not empty row.
	 */
	public boolean hasNext() {
		return next;
	}

	/**
	 * 
	 * @return the next not empty row.
	 */
	public Row next() {
		if (!hasNext()) {
			throw new NoSuchElementException();
		}
		Row retour = currentRow;
		ReaderException err = this.exception;
		try {
			currentRow = setNext();
		} catch (ReaderException e) {
			this.exception = e;
		}
		if (err != null) {

			throw err;
		}

		return retour;
	}

	/**
	 * 
	 * @return the next not empty row.
	 */
	private Row setNext() {
		Row retour = null;

		do {
			if (reader.hasNext()) {
				retour = reader.next();
				next = true;
			} else {
				next = false;
				retour = null;
			}
		} while (retour != null && retour.isEmpty());
		return retour;
	}

	/**
	 * Throws UnsupportedOperationException.
	 */
	public void remove() {
		throw new UnsupportedOperationException();

	}

	/**
	 * close the original reader.
	 */
	public void close() {
		try {
			reader.close();
		} catch (IOException e) {
			LOGGER.info("Error on closing reader", e);
		}
	}

	/**
	 * @return the original withHeaders attribute.
	 */
	public boolean isWithHeaders() {
		return reader.isWithHeaders();
	}

}
