/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Util in loading objects from XML Documents and identifying getter and setter
 * method.
 * 
 * @author Olivier Godineau
 * 
 */
public final class Util { // NOPMD by olivier on 28/01/12 14:39

	/**
	 * the class logger.
	 */
	private static final Logger LOGGER = LoggerFactory.getLogger(Util.class);

	/**
	 * private constructor to prevent its instantiation.
	 */
	private Util() {

	}
	/**
	 * Returns Class corresponding to the given field identify by its setter.
	 * 
	 * @param clazz
	 *            class which has the given field
	 * @param field
	 *            field name
	 * @param <T>
	 *            the type of the class
	 * @return declared class by the setter as the returned type
	 * @throws NoSuchMethodException
	 *             if no setter found
	 */
	public static <T> Class<?> identifySetType(Class<T> clazz, String field) throws NoSuchMethodException {

		Method setter = identifySetter(clazz, field);

		if (setter == null) {
			throw new NoSuchMethodException(field + " has no setter");
		}
		return setter.getParameterTypes()[0];

	}

	/**
	 * Returns public setter corresponding to the given field.
	 * 
	 * @param clazz
	 *            class which has the given field
	 * @param field
	 *            field name
	 * @param <T>
	 *            the type of the class.
	 * @return <code>null</code> if no public setter found
	 */
	public static <T> Method identifySetter(Class<T> clazz, String field) {
		Method method = null;
		if (clazz != null && field != null) {
			try {
				BeanInfo info = Introspector.getBeanInfo(clazz);
				for (PropertyDescriptor pd : info.getPropertyDescriptors()) {
					if (field.equals(pd.getName())) {
						method = pd.getWriteMethod();
						break;
					}
				}
			} catch (IntrospectionException e) {
				LOGGER.error(String.format("%s.%s setter search : ", clazz.getSimpleName(), field), e);
			}
		}
		return method;

	}

	/**
	 * Returns public getter corresponding to the given field.
	 * 
	 * @param clazz
	 *            class which has the given field
	 * @param field
	 *            field name
	 * @param <T>
	 *            the type of the class.
	 * @return <code>null</code> if no public getter found
	 */

	public static <T> Method identifyGetter(Class<T> clazz, String field) {
		Method method = null;
		if (clazz != null && field != null) {
			try {
				BeanInfo info = Introspector.getBeanInfo(clazz);
				for (PropertyDescriptor pd : info.getPropertyDescriptors()) {
					if (field.equals(pd.getName())) {
						method = pd.getReadMethod();
						break;
					}
				}
			} catch (IntrospectionException e) {
				LOGGER.error(String.format("%s.%s getter search : ", clazz.getSimpleName(), field), e);
			}
		}
		return method;

	}

	/**
	 * Convert empty String to <code>null</code>.
	 * 
	 * @param str
	 *            String to convert
	 * @return the converted value.
	 */
	public static String emptyToNull(String str) {
		String retour = null;
		if (!"".equals(str)) {
			retour = str;
		}
		return retour;
	}

	/**
	 * Definie if a class is a subclass of an another class.
	 * 
	 * @param clazzA
	 *            the class.
	 * @param clazzB
	 *            the subclass candidate.
	 * @return true if clazzB is a subclass of clazzA.
	 */
	public static boolean asSubClass(Class<?> clazzA, Class<?> clazzB) {
		try {
			clazzB.asSubclass(clazzA);
			return true;
		} catch (ClassCastException ex) {
			// Nothing
		}
		return false;
	}

	/**
	 * Returns if a class is concrete and not an enum class.
	 * 
	 * @param clazz
	 *            the class.
	 * @param <T>
	 *            the type of the class.
	 * @return true if the class is concrete.
	 */
	public static <T> boolean isConcrete(Class<T> clazz) {
		return clazz.isPrimitive() || clazz.isEnum()
				|| (!clazz.isInterface() && !Modifier.isAbstract(clazz.getModifiers()));
	}

}
