/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.impl;

import olg.csv.base.Cell;
import olg.csv.bean.ICellProcessor;

/**
 * This class allows to format a column cell from entity properties values.
 * 
 * @author Olivier Godineau
 * 
 * @param <B>
 *            the entity type
 * 
 */
public final class CellProcessor<B> implements Comparable<CellProcessor<B>>, ICellProcessor<B> {
	/**
	 * column number.
	 */
	private int rang;

	/**
	 * column header name.
	 */
	private String name;

	/**
	 * formatter to rendering the entity properties as value of a cell.
	 */
	private PropertyFormatter propertyFormatter;

	/**
	 * 
	 * @param rang
	 *            column number
	 * @param name
	 *            column header name
	 * @param propertyFormatter
	 *            the formatter to rendering the entity properties as value of
	 *            column cell
	 */
	public CellProcessor(String rang, String name, PropertyFormatter propertyFormatter) {
		super();
		this.rang = Cell.fromSheetCellNumber(rang);
		this.name = name;
		this.propertyFormatter = propertyFormatter;
	}

	/**
	 * Returns the column num. Column num begin at 0.
	 * 
	 * @return the columne num.
	 */
	public int getRang() {
		return rang;
	}

	/**
	 * Get the column header name.
	 * 
	 * @return the column header name.
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the formatter dedicated to rendering the entity properties as
	 * value of column cell.
	 * 
	 * @return the formatter.
	 */
	public PropertyFormatter getPropertyFormatter() {
		return propertyFormatter;
	}

	/**
	 * Sets the column num.
	 * 
	 * @param rang
	 *            the column num.
	 */
	public void setRang(int rang) {
		this.rang = rang;
	}

	/**
	 * Set the column header name.
	 * 
	 * @param name
	 *            the header.
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Sets the formatter dedicated to format entity properties as cell value.
	 * 
	 * @param propertyFormatter
	 *            the formatter.
	 */
	public void setPropertyFormatter(PropertyFormatter propertyFormatter) {
		this.propertyFormatter = propertyFormatter;
	}
	/**
	 * {@inheritDoc} Comparison is based on the rang field.
	 */
	public int compareTo(CellProcessor<B> columnFormatter) {
		return this.rang - columnFormatter.rang;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((propertyFormatter == null) ? 0 : propertyFormatter.hashCode());
		result = prime * result + rang;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		CellProcessor<?> other = (CellProcessor<?>) obj;
		if (name == null) {
			if (other.name != null) {
				return false;
			}
		} else if (!name.equals(other.name)) {
			return false;
		}
		if (propertyFormatter == null) {
			if (other.propertyFormatter != null) {
				return false;
			}
		} else if (!propertyFormatter.equals(other.propertyFormatter)) {
			return false;
		}
		if (rang != other.rang) {
			return false;
		}
		return true;
	}

	/**
	 * {@inheritDoc}
	 */
	public Cell transform(B e) {
		return new Cell(this.getRang(), this.getPropertyFormatter().toString(e));
	}

	/**
	 * {@inheritDoc}
	 */
	public Cell getHeader() {
		return new Cell(this.getRang(), this.getName());
	}
}
