/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader.getter;

import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import olg.csv.bean.getter.AbstractGetter;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.loader.Util;
import olg.csv.bean.loader.filter.AbstractFiltreLoader;
import olg.csv.bean.loader.getter.impl.ConcateGetterLoader;
import olg.csv.bean.loader.getter.impl.ConstantGetterLoader;
import olg.csv.bean.loader.getter.impl.CustomGetterLoader;
import olg.csv.bean.loader.getter.impl.DefaultGetterLoader;

import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Class dedicated to load a getter {@link AbstractGetter} from an XML element
 * conformed to XML schema GetterType specification. Each concret class shall
 * have the responsibility to load a concrete AbstractGetter class.
 * 
 * <p>
 * Class based on Chain of responsability pattern and used to chain concret
 * Getter Loaders.
 * </p>
 * 
 * @author Olivier Godineau
 * @see AbstractGetter
 */
public abstract class AbstractGetterLoader {

	/**
	 * Singleton responsible of getters loading.
	 */
	private static AbstractGetterLoader instance = new DefaultGetterLoader(new ConcateGetterLoader(
			new ConstantGetterLoader(new CustomGetterLoader(null))));

	/**
	 * Returns singleton responsible of getters loading
	 * 
	 * This is this instance you have to be used to load every type of getter
	 * from XML element conformed to XML Schema GetterType specification.
	 * 
	 * @return the getters loader.
	 */
	public static AbstractGetterLoader getInstance() {
		return instance;
	}

	/**
	 * GetterLoader successor as describe in Chain of responsability pattern.
	 */
	protected AbstractGetterLoader successor;

	/**
	 * This Xpath expression allows to identify xml node which describe the
	 * corresponding concret filter under Fitler Type node.
	 */
	protected String xPathExpression;

	/**
	 * Returns a concret getter identified from the given XML element.
	 * 
	 * @param element
	 *            XML node corresponding to correspondant to XML GetterType as
	 *            described in our XML schema.
	 * @return a concret getter or <code>null</code> if the given element
	 *         doesn't match concret getter type this loader has responsability
	 *         and doesn't match any successor
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 */
	public final AbstractGetter getGetter(Element element) throws XPathExpressionException, LoadException {
		AbstractGetter retour = null;
		String defaultValue = null;
		if (element != null) {
			Attr attr = element.getAttributeNode("default");
			if (attr != null) {
				defaultValue = attr.getTextContent();
			}
			Node node = (Node) Util.evaluerDOM(element, xPathExpression, XPathConstants.NODE);
			if (node != null) { // NOPMD by olivier on 01/02/12 00:44
				retour = getConcreteGetter((Element) node, defaultValue);
				node = (Node) Util.evaluerDOM(element, "filter", XPathConstants.NODE);
				retour.setFilter(AbstractFiltreLoader.getInstance().getFilter((Element) node));

			} else {
				if (successor != null) {
					retour = successor.getGetter(element);
				}
			}
		}
		return retour;
	}

	/**
	 * Returns a concret getter.
	 * 
	 * @param node
	 *            XML Element corresponding to the XPath expression from which
	 *            the corresponding getter is loaded.
	 * @param defaultValue
	 *            value the loaded getter must used as default value.
	 * @return a concret getter this loader has responsability or
	 *         <code>null</code> if the given node doesn't match the type this
	 *         loader should return.
	 * @throws XPathExpressionException
	 *             on invalid XPathExpression
	 * @throws LoadException
	 *             on Error occurs during loading
	 * @see #xPathExpression
	 */
	protected abstract AbstractGetter getConcreteGetter(Element node, String defaultValue)
			throws XPathExpressionException, LoadException;

	/**
	 * 
	 * @param xPathExpression
	 *            XPath expression that allows to identify the concret getter
	 *            under the XML GetterType node
	 * @param successor
	 *            following concret loader in the chain of responsability
	 * 
	 */
	protected AbstractGetterLoader(String xPathExpression, AbstractGetterLoader successor) {
		super();
		this.successor = successor;
		this.xPathExpression = xPathExpression;
	}
}
