/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.parser;

import java.util.Date;
import java.util.Locale;

import olg.csv.base.UsageInvalideException;
import olg.csv.bean.parser.impl.CharacterParser;
import olg.csv.bean.parser.impl.ConstructorParser;
import olg.csv.bean.parser.impl.DateParser;
import olg.csv.bean.parser.impl.EnumParser;

/**
 * Class used to convert a string into object.
 * 
 * @author Olivier Godineau
 * 
 * @param <T>
 *            object this parser returns.
 */
public abstract class AbstractParser<T> {

	/**
	 * Converts a string into a new instance of <T>.
	 * 
	 * @param str
	 *            the string.
	 * @return the new instance of <T>.
	 * @throws ParseException
	 *             if parsing error occurs.
	 */
	public abstract T parse(String str) throws ParseException;

	/**
	 * Returns parser specialized in string conversion to date.
	 * 
	 * @param format
	 *            the date format.
	 * @param clazz
	 *            The class that the returned parser have to return. this class
	 *            must to have java.util.Date as superclass.
	 * @param locale
	 *            a Locale to apply. May be <code>null</code>
	 * @return a new date parser
	 */
	public static AbstractParser<Date> getDateParser(String format, Class<? extends Date> clazz, Locale locale) {

		return new DateParser(format, clazz, locale);

	}

	/**
	 * Returns the parser that operates from the constructor of the given class.
	 * Throws an UsageInvalideException if no such constructor exists or is
	 * visible.
	 * 
	 * @param clazz
	 *            The class that the returned parser have to return. This class
	 *            must have a constructor with string argument. This constructor
	 *            will be used to transform a String into a new Instance of this
	 *            class.
	 * @param <T>
	 *            the type the parser returns
	 * @return a new parser based on the string constructor of the given class
	 * 
	 */
	public static <T> AbstractParser<T> getConstructorParser(Class<T> clazz) {
		try {
			return new ConstructorParser<T>(clazz);
		} catch (SecurityException e) {
			throw new UsageInvalideException(e);

		} catch (NoSuchMethodException e) {
			throw new UsageInvalideException(e);
		}
	}

	/**
	 * Returns parser specialized in string conversion to Character or char.
	 * 
	 * @return a new character parser.
	 */
	public static AbstractParser<Character> getCharacterParser() {

		return new CharacterParser();
	}

	/**
	 * Returns parser specialized in string conversion to Enum.
	 * <p>
	 * this parser identifies the enum constant to return from a string seen as
	 * the enum constant name.
	 * </p>
	 * 
	 * @param clazz
	 *            Enum concret class
	 * @param <E>
	 *            the type the parser returns
	 * @return a new parser
	 */
	public static <E extends Enum<E>> AbstractParser<E> getEnumParser(Class<E> clazz) {

		return new EnumParser<E>(clazz);

	}

	/**
	 * Returns default parser corresponding to the given class. This parser
	 * would be responsible to convert a string into an object with the given
	 * class.
	 * <p>
	 * if the given class is not a primitive class, enum, or Character class,
	 * try to return a parser based on a constructor of this class accepting a
	 * string as argument.
	 * </p>
	 * <p>
	 * throws an UsageInvalideException In case the parser we are trying to
	 * return is based on class constructor and no such constructor exists or is
	 * visible.
	 * </p>
	 * 
	 * @param clazz
	 *            the class.
	 * @return the parser or <code>null</code> if none matches the class.
	 * 
	 * @see ConstructorParser
	 */
	@SuppressWarnings({"unchecked", "rawtypes"})
	public static AbstractParser identifyDefaultParser(Class clazz) {
		AbstractParser parser = null;
		if (clazz.isPrimitive()) {
			parser = PrimitiveType.getPrimitiveTypeByName(clazz.toString()).getParser();
		} else {
			if (clazz.isEnum()) {
				parser = AbstractParser.getEnumParser(clazz);
			} else {
				if (clazz.equals(Character.class)) {
					parser = AbstractParser.getCharacterParser();
				} else {
					parser = AbstractParser.getConstructorParser(clazz);
				}
			}
		}

		return parser;
	}

}
