/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.parser;

import olg.csv.bean.parser.impl.DecorateParser;

/**
 * Allows to associate a parser to a primitive type.
 * 
 */
enum PrimitiveType {
	/**
	 * Boolean type. "false" is returned as null or empty string is parsed.
	 */
	booleanType("boolean", new DecorateParser<Boolean>(AbstractParser.getConstructorParser(Boolean.class), "false")),
	/**
	 * Byte type. 0 is returned as null or empty string is parsed.
	 */
	byteType("byte", new DecorateParser<Byte>(AbstractParser.getConstructorParser(Byte.class), "0")),
	/**
	 * Short type. 0 is returned as null or empty string is parsed.
	 */
	shortType("short", new DecorateParser<Short>(AbstractParser.getConstructorParser(Short.class), "0")),
	/**
	 * Integer type. 0 is returned as null or empty string is parsed.
	 */
	intType("int", new DecorateParser<Integer>(AbstractParser.getConstructorParser(Integer.class), "0")),
	/**
	 * Long type. 0 is returned as null or empty string is parsed.
	 */
	longType("long", new DecorateParser<Long>(AbstractParser.getConstructorParser(Long.class), "0")),
	/**
	 * Float type. 0 is returned as null or empty string is parsed.
	 */
	floatType("float", new DecorateParser<Float>(AbstractParser.getConstructorParser(Float.class), "0")),

	/**
	 * Double type. 0 is returned as null or empty string is parsed.
	 */
	doubleType("double", new DecorateParser<Double>(AbstractParser.getConstructorParser(Double.class), "0")),
	/**
	 * Character type. \u0000 is returned as null or empty string is parsed.
	 */
	charType("char", new DecorateParser<Character>(AbstractParser.getCharacterParser(), "\u0000"));

	/**
	 * the name of the primitive.
	 */
	private String primitiveName;

	/**
	 * the parser to use.
	 */
	private AbstractParser<?> parser;

	/**
	 * Constructor.
	 * 
	 * @param primitiveName
	 *            the primitive name
	 * @param parser
	 *            the parser
	 */
	private PrimitiveType(String primitiveName, AbstractParser<?> parser) {

		this.primitiveName = primitiveName;
		this.parser = parser;

	}

	/**
	 * The parse associated with the primitive type this enum matches.
	 * 
	 * @return the parser matching the primitive type.
	 */
	public AbstractParser<?> getParser() {
		return parser;
	}

	/**
	 * Returns the enum value identify by its name.
	 * 
	 * @param name
	 *            nom du type primitif.
	 * @return <code>null</code> if the given name doesn't match any enum value.
	 */
	public static PrimitiveType getPrimitiveTypeByName(String name) {
		PrimitiveType retour = null;
		for (PrimitiveType type : PrimitiveType.values()) {
			if (type.primitiveName.equals(name)) {
				retour = type;
				break;
			}
		}
		return retour;
	}
}