/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base;

import java.io.File;
import java.io.IOException;
import java.util.NoSuchElementException;

import olg.csv.base.csv.CSVReader;
import olg.csv.base.csv.CSVSettings;
import olg.csv.base.ods.ODSReader;
import olg.csv.base.ods.ODSSettings;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class IReaderTestCase extends AbstractReaderTestCase {
	private static final Logger LOGGER = LoggerFactory.getLogger(IReaderTestCase.class);

	private enum Type {
		CSV,
		// XLS,
		ODS;
		IReader getReader(Type type, File file) throws IOException {
			IReader result = null;
			switch (type) {
				case CSV :
					CSVSettings settings = new CSVSettings('\"', ',', false).setCharsetName("US-ASCII");
					result = new CSVReader(file, settings);
					break;
				// case XLS :
				// result = new XLSReader(f, 0, false);
				// break;
				case ODS :
					result = new ODSReader(file, new ODSSettings(0, null, null, null, null, false));
					break;
				default :
					throw new IllegalArgumentException("Unsupported type :" + type);
			}
			return result;

		}

	}

	private void noEmptyRowCase(IReader reader) {

		Row expected = getTemplate(2, "xxx", "yyy", "zzz");
		assertTrue("first row expected", reader.hasNext());
		// 1st
		// xxx,yyy,zzz
		Row row = reader.next();
		assertEqualsRow(expected, row);

		assertTrue("2nd row expected", reader.hasNext());
		// 2nd
		// "xxx","yyy","zzz"
		row = reader.next();
		expected = getTemplate(5, "xxx", "yyy", "zzz");
		assertEqualsRow(expected, row);

		assertFalse("no row expected", reader.hasNext());

		try {
			reader.next();
			fail();
		} catch (NoSuchElementException ex) {
			LOGGER.debug("Expected error on next method calling", ex);
		}

	}

	public void testNoEmptyRow() {
		File file = null;
		for (Type type : Type.values()) {
			LOGGER.debug(type + "NoEmptyRow Case");

			file = new File( // NOPMD by olivier on 26/01/12 01:07
					this.getClass()
							.getResource(
									IReaderTestCase.class.getSimpleName() + "-testNoEmptyRow."
											+ type.name().toLowerCase()).getFile());
			IReader reader = null;
			try {
				reader = new IgnoreNullReader(type.getReader(type, file)); // NOPMD
																			// by
																			// olivier
																			// on
																			// 26/01/12
																			// 01:08
				noEmptyRowCase(reader);
			} catch (IOException e) {
				fail("fail on " + type + "Reader instanciation :" + e.toString());

			}

		}
	}

	public void testWithNoFile() {
		File file = null;
		for (Type type : Type.values()) {
			LOGGER.debug(type + "WithNoFile Case");
			file = new File("nofile"); // NOPMD by olivier on 26/01/12 01:08
			IReader reader = null;
			try {
				reader = type.getReader(type, file);
				fail(type + "Reader instanciation must fail on loading a file not found");
			} catch (IOException e) {
				LOGGER.debug("Expected Error on  reader instanciation", e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {
						LOGGER.debug("Error on closing reader", e);
					}
				}

			}
		}

	}

	public void testWithEmptyFile() {

		// Test for ODS Empty file is KO because ODS file cannot be empty : a
		// sheet has at least one row, and one column!
		Type type = Type.CSV;
		LOGGER.debug(type + "WithEmptyFile Case");
		File file = new File(this.getClass()
				.getResource(IReaderTestCase.class.getSimpleName() + "-testWithEmptyFile." + type.name().toLowerCase())
				.getFile());
		IReader reader = null;
		try {
			reader = type.getReader(type, file);

			assertFalse(type + "Reader.hasNext false expected", reader.hasNext());
		} catch (IOException e) {
			fail(type + ":" + e.toString());
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (IOException e) {
					LOGGER.debug("Error on closing reader", e);
				}
			}

		}

	}

}
