/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;
import olg.csv.base.csv.CSVReader;
import olg.csv.base.csv.CSVSettings;
import olg.csv.base.csv.CSVWriter;
import olg.csv.base.ods.ODSReader;
import olg.csv.base.ods.ODSSettings;
import olg.csv.base.ods.ODSWriter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class IWriterTestCase extends TestCase {
	private static final Logger LOGGER = LoggerFactory.getLogger(IWriterTestCase.class);

	private enum Type {
		CSV,
		// XLS,
		ODS;
		static IReader getReader(Type type, File file) throws IOException {
			IReader result = null;
			switch (type) {
				case CSV :
					result = new CSVReader(file, new CSVSettings('\"', ',', false).setLineSeparator("\r\n")
							.setCharsetName("UTF-8"));
					break;

				case ODS :
					result = new ODSReader(file, new ODSSettings(0, null, null, null, null, false));
					break;
				default :
					throw new IllegalArgumentException("Unsupported type :" + type);
			}
			return result;

		}
		static IWriter getWriter(Type type, File file) throws IOException {
			IWriter result = null;
			switch (type) {
				case CSV :
					result = new CSVWriter(file, new CSVSettings('\"', ',', true).setLineSeparator("\r\n")
							.setCharsetName("UTF-8"));
					break;

				case ODS :
					result = new ODSWriter(file, new ODSSettings("ODSWriter test", null, null));
					break;
				default :
					throw new IllegalArgumentException("Unsupported type :" + type);
			}
			return result;
		}
	};

	public void testInvalid() {
		File file = null;
		for (Type type : Type.values()) {
			file = new File("src/test/resources/olg/csv/bean", IWriterTestCase.class.getSimpleName() + "-testInvalid."
					+ type.name().toLowerCase()); // NOPMD by olivier on
													// 31/01/12 01:01
			try {
				if (file.isFile()) {
					if (!file.delete()) {
						LOGGER.warn("Unable to delete file {}", file);
					}
				}
				LOGGER.debug(type + "Writer invalidCase");
				IWriter writer = Type.getWriter(type, file);
				try {
					invalidCase(writer);
				} finally {

					try {
						writer.close();
					} catch (IOException e) {
						LOGGER.debug("Error on closing writer", e);
					}

					if (!file.delete()) {
						LOGGER.warn("Unable to delete file {}", file);
					}
				}
			} catch (IOException e) {
				fail(e.toString());
				LOGGER.debug("Unexpected Error on invalidCase test", e);
			}
		}

	}

	public void testValid() {

		IWriter writer = null;
		IReader reader = null;
		for (Type type : Type.values()) {
			File file = new File("src/test/resources/olg/csv/bean", IWriterTestCase.class.getSimpleName()
					+ "-testValid." + type.name().toLowerCase()); // NOPMD by
																	// olivier
																	// on
																	// 31/01/12
																	// 01:01
			try {
				if (file.isFile()) {
					if (!file.delete()) {
						LOGGER.warn("Unable to delete file {}", file);
					}
				}
				LOGGER.debug(type + "Writer validCase");
				writer = Type.getWriter(type, file);
				valideCase(writer);
			} catch (IOException e) {
				fail(e.toString());
				LOGGER.debug("Unexpected Error on validCase test", e);
			} finally {
				if (writer != null) {
					try {
						writer.close();
					} catch (IOException e) {
						LOGGER.debug("Error on closing writer", e);
					}
				}
			}

			try {
				reader = Type.getReader(type, file);
				checkValidCase(reader);
			} catch (IOException e) {
				fail(e.toString());
				LOGGER.debug("Unexpected Error on valideCase test", e);
			} finally {
				if (reader != null) {
					try {
						reader.close();
					} catch (IOException e) {
						LOGGER.debug("Error on closing reader", e);
					}
				}
			}
		}
	}

	private void invalidCase(IWriter writer) {
		String ErrorMessage = "Expected Error on invalideCase test";

		try {
			writer.addLine(null);
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}
		try {
			writer.addRow(null);
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}

		try {
			writer.addLine(null);
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}
		List<Cell> cells = new ArrayList<Cell>();
		cells.add(new Cell(0, "xxx"));
		cells.add(new Cell(1, "yyy"));
		cells.add(new Cell(2, "zzz"));
		Row row = new Row(1, cells, cells.size());
		writer.addRow(row);
		try {
			writer.addRow(new Row(2, cells, cells.size() + 1));
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}
		try {
			writer.addRow(new Row(2, cells, cells.size() - 1));
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}
		writer.addRow(new Row(4, cells, cells.size()));
		try {
			writer.addRow(new Row(3, cells, cells.size()));
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}
		try {
			writer.addLine(new String[]{"xxx", "yyy"});
			fail();
		} catch (Exception ex) {
			LOGGER.debug(ErrorMessage, ex);
		}

	}

	private void valideCase(IWriter writer) {

		Row row = new Row(1, null, 3);
		writer.addRow(row);
		List<Cell> cells = new ArrayList<Cell>();

		cells.add(new Cell(0, ""));
		cells.add(new Cell(1, null));
		cells.add(new Cell(2, null));
		row = new Row(4, cells, cells.size());
		writer.addRow(row);
		writer.addLine(new String[]{"", null, null});

		cells = new ArrayList<Cell>();
		cells.add(new Cell(1, "yyy"));
		row = new Row(6, cells, 3);
		writer.addRow(row);

		writer.addLine(new String[]{"xxx", "yyy", "zzz"});

		cells = new ArrayList<Cell>();
		cells.add(new Cell(0, "xxx"));
		cells.add(new Cell(1, "yyy"));
		cells.add(new Cell(2, "zzz"));
		row = new Row(8, cells, 3);
		writer.addRow(row);

		cells = new ArrayList<Cell>();
		cells.add(new Cell(0, "xxx\",\","));
		cells.add(new Cell(1, "y\"yy"));
		cells.add(new Cell(2, "zzz\"\ntralala"));
		row = new Row(9, cells, cells.size());
		writer.addRow(row);

	}

	private void checkValidCase(IReader reader) {
		for (int i = 0; i < 5; i++) {
			assertTrue("hasNext", reader.hasNext());
			Row row = reader.next();
			assertEquals("Row size", 3, row.getSize());
			assertEquals(i + 1, row.getNum());
			for (Cell cell : row) {
				assertNull("cell not null", cell.getValue());
			}
		}
		assertTrue(reader.hasNext());
		Row row = reader.next();
		assertEquals(3, row.getSize());
		assertEquals(6, row.getNum());
		assertEquals("Cell value", "yyy", row.getCell(1).getValue());
		assertNull("Cell not null", row.getCell(0).getValue());
		assertNull("Cell not null", row.getCell(2).getValue());

		assertTrue(reader.hasNext());
		row = reader.next();
		assertEquals(3, row.getSize());
		assertEquals(7, row.getNum());
		assertEquals("Cell value error", "xxx", row.getCell(0).getValue());
		assertEquals("Cell value error", "yyy", row.getCell(1).getValue());
		assertEquals("Cell value error", "zzz", row.getCell(2).getValue());
		assertTrue(reader.hasNext());
		row = reader.next();
		assertEquals(3, row.getSize());
		assertEquals(8, row.getNum());
		assertEquals("Cell value error", "xxx", row.getCell(0).getValue());
		assertEquals("Cell value error", "yyy", row.getCell(1).getValue());
		assertEquals("Cell value error", "zzz", row.getCell(2).getValue());
		assertTrue(reader.hasNext());
		row = reader.next();
		assertEquals(3, row.getSize());
		assertEquals(9, row.getNum());
		assertEquals("Cell value error", "xxx\",\",", row.getCell(0).getValue());
		assertEquals("Cell value error", "y\"yy", row.getCell(1).getValue());
		assertEquals("Cell value error", "zzz\"\ntralala", row.getCell(2).getValue());
		assertFalse("end of file error", reader.hasNext());

	}

}
