/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.csv;

import java.io.File;
import java.io.IOException;
import java.util.NoSuchElementException;

import olg.csv.base.AbstractReaderTestCase;
import olg.csv.base.IReader;
import olg.csv.base.ReaderException;
import olg.csv.base.Row;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class CSVReaderTestCase extends AbstractReaderTestCase {

	private static final Logger LOGGER = LoggerFactory.getLogger(CSVReaderTestCase.class);

	private void doTestValid(IReader reader, boolean skipEmptyCell, String delimiter, String separator) {

		try {

			assertTrue("first row expected", reader.hasNext());
			Row expected = getTemplate(1, "xxx", "yyy", "zzz");
			// 1st
			// xxx,yyy,zzz
			Row row = reader.next();
			assertEqualsRow(expected, row);

			assertTrue("2nd row expected", reader.hasNext());
			// 2nd
			// "xxx","yyy","zzz"
			row = reader.next();
			expected = getTemplate(2, "xxx", "yyy", "zzz");
			assertEqualsRow(expected, row);

			assertTrue("3rd row expected", reader.hasNext());

			// 3rd
			// xxx,"yyy",zzz
			row = reader.next();
			expected = getTemplate(3, "xxx", "yyy", "zzz");
			assertEqualsRow(expected, row);
			assertTrue("4th row expected", reader.hasNext());
			// 4th
			// ,,
			row = reader.next();
			assertEmptyRow(row, 4, skipEmptyCell);

			assertTrue("5th row expected", reader.hasNext());
			// 5th
			// "","",""
			row = reader.next();
			assertEmptyRow(row, 5, skipEmptyCell);

			assertTrue("6th row expected", reader.hasNext());
			// 6
			// "xxx"","",","y""yy","zzz""
			// "
			row = reader.next();
			expected = getTemplate(6, "xxx" + delimiter + separator + delimiter + separator, "y" + delimiter + "yy",
					"zzz" + delimiter + "\r\n");
			assertEqualsRow(expected, row);

			assertFalse("No more row expected", reader.hasNext());

			try {
				reader.next();
				fail();
			} catch (NoSuchElementException ex) {
				LOGGER.debug("Expected exception", ex);
			}

		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (Exception ex) {
					fail(ex.toString());
				}
			}
		}
	}

	public void testValid() {
		File file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid.csv")
				.getFile());

		try {
			CSVReader reader = new CSVReader(file, new CSVSettings());
			doTestValid(reader, false, "\"", ",");
		} catch (IOException e) {
			fail(e.toString());
		}

		try {
			CSVSettings settings = new CSVSettings().setCharsetName("US-ASCII").setSkipEmptyCell(true);
			CSVReader reader = new CSVReader(file, settings);

			doTestValid(reader, true, "\"", ",");
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	public void testValid_CR() {
		File file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid_CR.csv")
				.getFile());

		try {
			CSVReader reader = new CSVReader(file, new CSVSettings());
			doTestValid(reader, false, "\"", ",");
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	public void testValid_LF() {
		File file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid_LF.csv")
				.getFile());

		try {
			CSVReader reader = new CSVReader(file, new CSVSettings());
			doTestValid(reader, false, "\"", ",");
		} catch (IOException e) {
			fail(e.toString());
		}
	}
	public void testValid_CR_EOF() {
		File file = new File(this.getClass()
				.getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid_CR-EOF.csv").getFile());

		try {
			CSVReader reader = new CSVReader(file, new CSVSettings());
			doTestValid(reader, false, "\"", ",");
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	public void testValid_LF_EOF() {
		File file = new File(this.getClass()
				.getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid_LF_EOF.csv").getFile());

		try {
			CSVReader reader = new CSVReader(file, new CSVSettings());
			doTestValid(reader, false, "\"", ",");
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	public void testValid_BufSize() {
		File file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid.csv")
				.getFile());
		for (int bufSize = 1; bufSize < 200; bufSize++) {
			try {

				CSVReader reader = new CSVReader(file, new CSVSettings());
				doTestValid(reader, false, "\"", ",");

			} catch (IOException e) {
				fail(e.toString());
			}

		}
		file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testValid_LF_EOF.csv")
				.getFile());
		for (int bufSize = 1; bufSize < 200; bufSize++) {
			try {

				CSVReader reader = new CSVReader(file, new CSVSettings());
				doTestValid(reader, false, "\"", ",");

			} catch (IOException e) {
				fail(e.toString());
			}

		}

	}

	public void testInvalid() {
		File file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testInvalid.csv")
				.getFile());
		// permissive case
		CSVReader reader = null;
		try {
			reader = new CSVReader(file, new CSVSettings());
			assertTrue("Next row expected", reader.hasNext());
			assertNotNull("first row expected", reader.next());
			assertTrue("first row has a sucessor", reader.hasNext());
			try {

				reader.next();
				fail("retreiving next row should have failed");
			} catch (ReaderException ex) {
				LOGGER.debug("Expected exception on next operation", ex);
			}

		} catch (IOException e) {
			fail(e.toString());
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (Exception ex) {
					fail(ex.toString());
				}
			}
		}
		file = new File(this.getClass().getResource(CSVReaderTestCase.class.getSimpleName() + "-testInvalid2.csv")
				.getFile());

		try {
			reader = new CSVReader(file, new CSVSettings());
			assertTrue("First row expected", reader.hasNext());
			assertNotNull("Not null row expected", reader.next());
			assertTrue("2nd row row expected", reader.hasNext());
			try {
				reader.next();
				fail();
			} catch (ReaderException ex) {
				LOGGER.debug("Expected exception on next operation", ex);
			}

		} catch (IOException e) {
			fail(e.toString());
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch (Exception ex) {
					fail(ex.toString());
				}
			}
		}
	}

	public void testAlternateValid() {
		File file = new File(this.getClass()
				.getResource(CSVReaderTestCase.class.getSimpleName() + "-testAlternateValid.csv").getFile());
		// permissive case
		try {
			CSVSettings settings = new CSVSettings('\'', ';', false).setCharsetName("US-ASCII");
			CSVReader reader = new CSVReader(file, settings);
			doTestValid(reader, false, "\'", ";");
		} catch (IOException e) {
			fail(e.toString());
		}
	}

	public void testCharsetUTF8() {
		File file = new File(this.getClass()
				.getResource(CSVReaderTestCase.class.getSimpleName() + "-testEncodage-UTF8.csv").getFile());
		CSVSettings settings = new CSVSettings('"', ';', false).setCharsetName("UTF8");
		CSVReader reader = null;
		try {

			reader = new CSVReader(file, settings);
			Row row = reader.next();
			assertEquals("àéèçïîôöù", row.getCell(0).getValue());
		} catch (IOException e) {
			fail(e.toString());
		} finally {
			if (reader != null) {
				reader.close();
			}
		}
	}

	public void testCharsetISO8859() {
		File file = new File(this.getClass()
				.getResource(CSVReaderTestCase.class.getSimpleName() + "-testEncodage-ISO8859-1.csv").getFile());
		CSVSettings settings = new CSVSettings('"', ';', false).setCharsetName("ISO-8859-1");
		CSVReader reader = null;
		try {

			reader = new CSVReader(file, settings);
			Row row = reader.next();
			assertEquals("àéèçïîôöù", row.getCell(0).getValue());
		} catch (IOException e) {
			fail(e.toString());
		} finally {
			if (reader != null) {
				reader.close();
			}
		}
	}

}
