/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.base.ods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;
import olg.csv.base.Cell;
import olg.csv.base.Row;
import olg.csv.base.ods.ODSWriter.ODSParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class ODSWriterTestCase extends TestCase {
	private static final Logger LOGGER = LoggerFactory.getLogger(ODSWriterTestCase.class);

	private static final String DEFAULT_CELLVALUE = "AAA";
	private static final String SHEETNAME = "Sheet test";

	/**
	 * ODSParser test
	 * 
	 * @see ODSParser
	 */
	public void testODSParser() {
		ODSWriter.ODSParser parser = new ODSWriter.ODSParser();
		List<String> expected = Arrays.asList(new String[]{"one", "two", "three"});

		assertEquals("Error with \\r line separator", expected, parser.parse("one\rtwo\rthree"));
		assertEquals("Error with \\n line separator", expected, parser.parse("one\ntwo\nthree"));
		assertEquals("Error with \\r\\n line separator", expected, parser.parse("one\r\ntwo\r\nthree"));
		assertEquals("Error with \\r or \\r\\n line separator", expected, parser.parse("one\rtwo\r\nthree"));

		expected = Arrays.asList(new String[]{"one", "", "two", "", "", "three"});
		String errorMessage = "error on blank line production";
		assertEquals(errorMessage, expected, parser.parse("one\r\rtwo\r\r\rthree"));
		assertEquals(errorMessage, expected, parser.parse("one\r\r\ntwo\r\r\n\rthree"));
		assertEquals(errorMessage, expected, parser.parse("one\r\n\r\ntwo\r\n\r\n\r\nthree"));
		assertEquals(errorMessage, expected, parser.parse("one\n\ntwo\n\n\nthree"));

		expected = Arrays.asList(new String[]{"", "", "one", "", "two", "three", "", ""});
		assertEquals(errorMessage, expected, parser.parse("\r\rone\r\rtwo\rthree\r\r"));
		assertEquals(errorMessage, expected, parser.parse("\r\n\none\r\n\r\ntwo\r\nthree\r\n\r\n"));
		assertEquals(errorMessage, expected, parser.parse("\n\none\n\ntwo\nthree\n\n"));

	}

	private void rewrite(File file, ODSSettings settings) {
		ODSWriter writer = null;

		try {
			writer = new ODSWriter(file, settings);
			List<Cell> cells = new ArrayList<Cell>();
			cells.add(new Cell(0, "xxx"));
			cells.add(new Cell(1, "yyy"));
			cells.add(new Cell(4, "zzz"));

			writer.addRow(new Row(1, cells, 5));
			cells.clear();
			cells.add(new Cell(1, "xxx"));
			cells.add(new Cell(2, "yyy"));
			cells.add(new Cell(3, "zzz"));
			writer.addRow(new Row(3, cells, 5));

		} catch (IOException e) {
			fail(e.toString());

		} finally {
			if (writer != null) {
				writer.close();
			}
		}
	}

	private void assertODSRewriting(File file, ODSSettings settings) {
		ODSReader reader = null;
		try {
			String defaultCell = (settings.isEmptyingCell() ? null : DEFAULT_CELLVALUE);
			String cellDefaultRow = (settings.isEmptyingRow() ? null : DEFAULT_CELLVALUE);
			reader = new ODSReader(file, settings);
			assertEquals("sheet name differs from the expected", settings.getSheetName(), reader.getSheetName());
			for (int i = 0; i < 4; i++) {
				assertTrue("Next row expected", reader.hasNext());
				Row expected = getOriginal(i + 1);
				Row row = reader.next();
				assertEqualsRow(expected, row);
			}

			assertTrue("5th row expected", reader.hasNext());
			assertEqualsRow(getTransformRow(5, "xxx", "yyy", defaultCell, defaultCell, "zzz"), reader.next());
			assertTrue("6th row expected", reader.hasNext());
			assertEqualsRow(
					getTransformRow(6, cellDefaultRow, cellDefaultRow, cellDefaultRow, cellDefaultRow, cellDefaultRow),
					reader.next());
			assertTrue("7th row expected", reader.hasNext());
			assertEqualsRow(getTransformRow(7, defaultCell, "xxx", "yyy", "zzz", defaultCell), reader.next());
			assertTrue("8th row expected", reader.hasNext());
			assertEqualsRow(getOriginal(8), reader.next());

		} catch (IOException e) {
			fail(e.toString());
		} finally {
			if (reader != null) {
				reader.close();
			}
		}
	}

	/**
	 * Overriding sheet test
	 */
	public void testOverrideFile() {
		File original = new File(this.getClass()
				.getResource(ODSWriterTestCase.class.getSimpleName() + "-testOverrideFile.ods").getFile());
		File file = new File(original.getParentFile(), ODSWriterTestCase.class.getSimpleName()
				+ "-testOverrideFile-1.ods");

		try {
			ODSSettings settings = new ODSSettings(SHEETNAME, 5, "E");
			settings.setWithHeaders(false);
			copy(original, file);
			rewrite(file, settings);
			settings = new ODSSettings(SHEETNAME, null, null);
			settings.setSheetNum(1);
			settings.setWithHeaders(false);

			assertODSRewriting(file, settings);

		} finally {
			file.delete();
		}

		try {
			ODSSettings settings = new ODSSettings(SHEETNAME, 5, "E");
			settings.setWithHeaders(false);
			settings.setEmptyingCell(false);
			settings.setEmptyingRow(false);
			copy(original, file);
			rewrite(file, settings);
			settings = new ODSSettings(SHEETNAME, null, null);
			settings.setSheetNum(1);
			settings.setWithHeaders(false);
			settings.setEmptyingCell(false);
			settings.setEmptyingRow(false);

			assertODSRewriting(file, settings);

		} finally {
			file.delete();
		}
	}

	private Row getOriginal(int num) {

		List<Cell> cells = new ArrayList<Cell>();
		for (int i = 0; i < 15; i++) {
			cells.add(new Cell(i, DEFAULT_CELLVALUE));

		}
		return new Row(num, cells, 15);

	}

	private Row getTransformRow(int num, String... fields) {

		List<Cell> cells = new ArrayList<Cell>();
		for (int i = 0; i < 4; i++) {
			cells.add(new Cell(i, DEFAULT_CELLVALUE));

		}
		for (int i = 0; i < 5; i++) {
			cells.add(new Cell(4 + i, fields[i]));
		}
		for (int i = 9; i < 15; i++) {
			cells.add(new Cell(i, DEFAULT_CELLVALUE));

		}

		return new Row(num, cells, 15);

	}

	protected void assertEqualsRow(Row expected, Row row) {
		if (expected == null) {
			assertNull("null row expected", row);

		} else {
			assertNotNull("not null row expected", row);
			assertEquals("row num differs from expected", expected.getNum(), row.getNum());
			assertEquals("row size differs from expected", expected.getSize(), row.getSize());
			for (int i = 0; i < row.getSize(); i++) {
				Cell expectedCell = expected.getCell(i);
				Cell cell = row.getCell(i);
				if (expectedCell == null) {
					assertNull("null cell expected", cell);
				} else {
					assertNotNull("not null cell expected", cell);
					assertEquals("cell num differs from expected", expectedCell.getNum(), cell.getNum());
					assertEquals("isEmpty cell method : value returned differs from expected", expectedCell.isEmpty(),
							cell.isEmpty());
					assertEquals("cell value differs from expected", expectedCell.getValue(), cell.getValue());
				}
			}

		}

	}

	private void copy(File src, File dest) {
		FileChannel srcChannel = null;
		FileChannel destChannel = null;
		try {
			srcChannel = new FileInputStream(src).getChannel();
			destChannel = new FileOutputStream(dest).getChannel();

			// Copie depuis le in vers le out
			srcChannel.transferTo(0, srcChannel.size(), destChannel);
		} catch (IOException e) {
			LOGGER.debug("Error on copying ", e);
		} finally {
			if (srcChannel != null) {
				try {
					srcChannel.close();
				} catch (IOException e) {
					LOGGER.debug("Error on closing channel", e);
				}
			}
			if (destChannel != null) {
				try {
					destChannel.close();
				} catch (IOException e) {
					LOGGER.debug("Error on closing channel", e);
				}
			}
		}
	}

}
