/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.impl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import olg.csv.base.Row;

import olg.csv.bean.IPropertyProcessor;
import olg.csv.bean.getter.AbstractGetter;
import olg.csv.bean.impl.AbstractPropertyProcessor;
import olg.csv.bean.impl.BeanProcessor;
import olg.csv.bean.parser.AbstractParser;
import olg.csv.bean.parser.ParseException;
import junit.framework.TestCase;

public class AbstractPropertyProcessorTestCase extends TestCase {
	private final static Logger LOGGER  = LoggerFactory.getLogger(AbstractPropertyProcessorTestCase.class);

	private static class Getter extends AbstractGetter {
		private String value;
	
	
		 Getter(String value) {
			super();
			this.value = value;
		}
		@Override
		protected String doGet(Row line) {
			return value;
		}

	}

	private static class Parser extends AbstractParser<String> {

		@Override
		public String parse(String str) throws ParseException {

			return str;
		}

		Parser() {
			super();

		}

	}

	public static class TestBean {
		private String field;

		private TestBean beanfield = null;

		public String getField() {
			return field;
		}

		public void setField(String field) {
			this.field = field;
		}

		public TestBean getBeanfield() {
			return beanfield;
		}

		public void setBeanfield(TestBean beanfield) {
			this.beanfield = beanfield;
		}

		

	}

	public void testSimplePropertyTransformer() {
		IPropertyProcessor<?> transformer = null;
		try {
			transformer = AbstractPropertyProcessor.getSimplePropertyTransformer(null, null,null,null);
			fail();
		} catch (IllegalArgumentException ex) { 
			LOGGER.debug("Expected IllegalArgumentException",ex);
		}
		try {
			transformer = AbstractPropertyProcessor.getSimplePropertyTransformer(new Getter("value"), null,null,null);
			fail();
		} catch (IllegalArgumentException ex) { 
			LOGGER.debug("Expected IllegalArgumentException",ex);
		}
		
		try {
			transformer = AbstractPropertyProcessor.getSimplePropertyTransformer(null,
					new Parser(), "field", TestBean.class);
			fail();
		} catch (IllegalArgumentException ex) { 
			LOGGER.debug("Expected IllegalArgumentException",ex);
		}
		
		transformer = AbstractPropertyProcessor.getSimplePropertyTransformer(new Getter("value"),
				new Parser(), "field", TestBean.class);

		try {
			TestBean bean = (TestBean) transformer.transform(new Row(1,null,5), null);
			assertNotNull("Not null TestBean expected",bean);
			assertEquals("TestBean field value differs from","value", bean.getField());// Les tests d'execution
			// concernent plutôt les
			// tests de la classe
			// BeanPropertyParser
		} catch (ParseException e) {
			LOGGER.error("Parse Error on testSimplePropertyTransformer", e);
			fail(e.toString());
		}
	}

	public void testComplexPropertyTransformer() {
		BeanProcessor<TestBean>  transformer= new BeanProcessor<TestBean> ( TestBean.class);
		transformer.add(AbstractPropertyProcessor.getSimplePropertyTransformer(new Getter("value"),
				new Parser(), "field", TestBean.class));
		IPropertyProcessor<TestBean> propertyTransformer = AbstractPropertyProcessor.getComplexPropertyTransformer(
				transformer, TestBean.class, "beanfield");
		try {
			TestBean bean =  propertyTransformer.transform(new Row(1,null,5), null);
			assertNotNull("bean must not be instanciate",bean);
			assertNotNull("bean.getBeanField() must return not null object",bean.getBeanfield());
			assertEquals("Beanfield.field value not match","value", bean.getBeanfield().getField());
			bean = new TestBean();
			bean.setField("test");
			TestBean bean2 =  propertyTransformer.transform(new Row(1,null,5), bean);
			assertNotNull("bean must be not null",bean2);
			assertEquals("bean and bean2 must be equal",bean, bean2);
			assertEquals("field must not be affected by beanfield transformer","test", bean2.getField());
			assertNotNull("After bean transforming, beanField must be not null",bean2.getBeanfield());
			assertEquals("Beanfield.field value not match","value", bean2.getBeanfield().getField());
		} catch (ParseException e) {
			LOGGER.error("Parse Error on testComplexPropertyTransformer", e);
			fail(e.toString());
		}
		try {
			propertyTransformer = AbstractPropertyProcessor.getComplexPropertyTransformer(transformer,
					TestBean.class, "noField");
			fail();
		} catch (IllegalArgumentException ex) { 
			LOGGER.debug("Expected IllegalArgumentException",ex);	
		}
		
		try {
			propertyTransformer = AbstractPropertyProcessor.getComplexPropertyTransformer(null, TestBean.class,
					"beanField");
			fail();
		} catch (IllegalArgumentException ex) { 	
			LOGGER.debug("Expected IllegalArgumentException",ex);
		}
		
		try {
			propertyTransformer = AbstractPropertyProcessor.getComplexPropertyTransformer(transformer, null,
					"noField");
			fail();
		} catch (IllegalArgumentException ex) { 
			LOGGER.debug("Expected IllegalArgumentException",ex);		
		}
		
		try {
			propertyTransformer = AbstractPropertyProcessor.getComplexPropertyTransformer(transformer,
					TestBean.class, null);
			fail();
		} catch (IllegalArgumentException ex) { 
			LOGGER.debug("Expected IllegalArgumentException",ex);	
		}
	}

}
