/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader.filter;

import java.io.IOException;
import java.util.Locale;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import junit.framework.TestCase;
import olg.csv.bean.TestUtil;
import olg.csv.bean.filter.AbstractStringFilter;
import olg.csv.bean.filter.impl.LowerCaseFilter;
import olg.csv.bean.filter.impl.UpperCaseFilter;
import olg.csv.bean.filter.impl.UpperCaseFirstFilter;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.loader.Util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * AbstractFilterLoader class TestCase
 * 
 * @author Olivier Godineau
 * 
 */
public class AbstractFiltreLoaderTestCase extends TestCase {
	private final static Logger LOGGER = LoggerFactory.getLogger(AbstractFiltreLoaderTestCase.class);

	public static class CustomNoFilter {
		public int getRang() {
			return rang;
		}
		public void setRang(int rang) {
			this.rang = rang;
		}
		public Float getTheFloat() {
			return theFloat;
		}
		public void setTheFloat(Float aFloat) {
			this.theFloat = aFloat;
		}
		private int rang;
		private Float theFloat;

	}

	public static class CustomFilter extends AbstractStringFilter {
		public int getRang() {
			return rang;
		}
		public void setRang(int rang) {
			this.rang = rang;
		}
		public Float getTheFloat() {
			return theFloat;
		}
		public void setTheFloat(Float aFloat) {
			this.theFloat = aFloat;
		}
		private int rang;
		private Float theFloat;

		@Override
		protected String doFiltre(String value) {

			return value + "tralala";
		}

	}

	private final String xmlContent = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +

	"<filter><upper-first locale=\"en\">\n" + "	<lower locale=\"en\">\n" + "		<upper locale=\"en\">\n" + "			<trim>\n"
			+ "				<split n=\"1\" separator=\"\\t\">\n" + "					<replace regex=\"\\\\\" replacement=\"/\">\n"
			+ "						<decorate before=\"because of\" >\n"
			+ "							<custom class=\"olg.csv.bean.loader.filter.AbstractFiltreLoaderTestCase$CustomFilter\">"
			+ "								<property name=\"rang\"><value>123</value></property>"
			+ "								<property name=\"theFloat\"><value>127.075</value></property>" + "								<trim/>"
			+ "							</custom>" + "						</decorate>\n" + "					</replace>\n" + "				</split>\n" + "			</trim>\n"
			+ "		</upper>\n" + "	</lower>\n" + "</upper-first>" + "</filter>";

	public void testFiltre() throws XPathExpressionException {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil.openDocument(xmlContent);
			element = (Element) Util.evaluerDOM(doc, "/filter", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			LOGGER.debug("Error on XML Parser configuration", e1);
			fail(e1.toString());
		} catch (SAXException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());
		} catch (IOException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());
		} catch (XPathExpressionException e) {
			LOGGER.debug("Error on XML Parsing", e);
			fail(e.toString());
		}

		try {
			AbstractStringFilter filtre = AbstractFiltreLoader.getInstance().getFilter(element);
			AbstractStringFilter successor = null;
			// le premier est UppercaseFirst
			assertTrue("The first filter expected is UpperCaseFirstFilter",
					filtre.getClass().getSimpleName().equals("UpperCaseFirstFilter"));
			assertNotNull("A chained filter is expected", filtre.getFilter());
			assertEquals("The first filter must have locale", Locale.ENGLISH,
					((UpperCaseFirstFilter) filtre).getLocale());
			// le second est Lowercase
			filtre = filtre.getFilter();
			assertTrue("The second filter expected is LowerCaseFilter",
					filtre.getClass().getSimpleName().equals("LowerCaseFilter"));
			assertEquals("The second filter must have locale", Locale.ENGLISH, ((LowerCaseFilter) filtre).getLocale());
			assertNotNull("A chained filter is expected", filtre.getFilter());

			// le troisième est Uppercase
			filtre = filtre.getFilter();
			assertTrue("The third filter expected is UpperCaseFilter",
					filtre.getClass().getSimpleName().equals("UpperCaseFilter"));
			assertNotNull("A chained filter is expected", filtre.getFilter());
			assertEquals("The third filter must have locale", Locale.ENGLISH, ((UpperCaseFilter) filtre).getLocale());

			// le 4ème est trim
			filtre = filtre.getFilter();
			assertTrue("The 4th filter expected is TrimFilter", filtre.getClass().getSimpleName().equals("TrimFilter"));
			assertNotNull("A chained filter is expected", filtre.getFilter());
			// le 5ème est split
			filtre = filtre.getFilter();
			assertNotNull("A chained filter is expected", filtre.getFilter());
			successor = filtre.getFilter();
			filtre.setFilter(null);
			assertTrue("The 5th filter expected is SplitFilter", filtre.getClass().getSimpleName()
					.equals("SplitFilter"));
			assertEquals("Split Filtering failed", "2", filtre.filtre("1\t2\t3"));

			// le 6ème est replace
			filtre = successor;
			assertNotNull("A chained filter is expected", filtre.getFilter());
			successor = filtre.getFilter();
			filtre.setFilter(null);
			assertTrue("The 6th filter expected is ReplaceFilter",
					filtre.getClass().getSimpleName().equals("ReplaceFilter"));
			assertEquals("Replace Filtering failed", "1/2/3", filtre.filtre("1\\2\\3"));

			// le 7ème est decorate
			filtre = successor;
			assertNotNull("A chained filter is expected", filtre.getFilter());
			successor = filtre.getFilter();
			filtre.setFilter(null);
			assertTrue("The 7th filter expected is DecorateFilter",
					filtre.getClass().getSimpleName().equals("DecorateFilter"));
			assertEquals("Decorate filtering failed", "because oftest", filtre.filtre("test"));

			// le 8ème est CustomFilter
			filtre = successor;
			assertNotNull("A chained filter is expected", filtre.getFilter());
			successor = filtre.getFilter();
			filtre.setFilter(null);
			assertTrue("The 8th filter expected is CustomFilter",
					filtre.getClass().getSimpleName().equals("CustomFilter"));
			CustomFilter customFilter = (CustomFilter) filtre;
			assertEquals("Setting customfilter property failed", 123, customFilter.getRang());
			assertEquals("Setting customfilter property failed", 127.075f, customFilter.getTheFloat());
			assertEquals("Custom Filtering failed", "un truc du type tralala", customFilter.filtre("un truc du type "));

			filtre = successor;
			assertTrue("The last filter must be TrimFilter", filtre.getClass().getSimpleName().equals("TrimFilter"));
			assertNull("The last chained filter must not have successor", filtre.getFilter());

		} catch (LoadException e) {
			LOGGER.debug("Error on Filter Loading", e);
			fail(e.toString());

		}
	}

	public void testFailedCustomFiltre() {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil
					.openDocument("<filter><custom class=\"olg.csv.bean.loader.filter.AbstractFiltreLoaderTestCase$CustomNoFilter\">"
							+ "								<property name=\"rang\"><value>123</value></property>"
							+ "								<property name=\"theFloat\"><value>127.075</value></property></custom>"
							+ "							</filter>\n");
			element = (Element) Util.evaluerDOM(doc, "/filter", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			LOGGER.debug("Error on XML Parser configuration", e1);
			fail(e1.toString());
		} catch (SAXException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());
		} catch (IOException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());
		} catch (XPathExpressionException e) {
			LOGGER.debug("Error on XML Parsing", e);
			fail(e.toString());
		}

		try {
			AbstractFiltreLoader.getInstance().getFilter(element);
		} catch (XPathExpressionException e) {
			LOGGER.debug("Error on Filter Loading", e);
			fail(e.toString());

		} catch (LoadException e) {
			LOGGER.debug("Error on Filter Loading", e);
			assertEquals("Error message differs from the expected", CustomNoFilter.class + " must extend "
					+ AbstractStringFilter.class, e.getMessage());

		}
	}
}
