/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader.getter;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import junit.framework.TestCase;
import olg.csv.base.Cell;
import olg.csv.base.Row;
import olg.csv.bean.TestUtil;
import olg.csv.bean.getter.AbstractGetter;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.loader.Util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * AbstractGetterLoader class TestCase
 * 
 * @author olivier
 * 
 */
public class AbstractGetterLoaderTestCase extends TestCase {
	private final static Logger LOGGER = LoggerFactory.getLogger(AbstractGetterLoaderTestCase.class);
	private final static String XML_CONSTANT_CONTENT = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +

	"<getter default=\"defaut\">" + "<constant></constant>" + "<filter>"
			+ "<decorate after=\"after\" before=\"before\"/>" + "</filter>" + "</getter>";

	private final static String XML_SIMPLE_CONTENT = "<getter default=\"defaut\"><column rang=\"1\"/>"
			+ "<filter><trim/></filter>" + "</getter>";

	private final static String XML_CUSTOM_CONTENT = "<getter default=\"defaut\"><custom class=\"olg.csv.bean.loader.getter.AbstractGetterLoaderTestCase$CustomGetter\">"
			+ "<property name=\"f\">"
			+ "<value>127.075</value>"
			+ "</property>"
			+ "</custom><filter><trim/></filter></getter>";

	/**
	 * Test class
	 * 
	 * @author Olivier Godineau
	 * 
	 */
	public static class CustomGetter extends AbstractGetter {
		private int rang;
		private Float f;
		public int getRang() {
			return rang;
		}
		public void setRang(int rang) {
			this.rang = rang;
		}
		public Float getF() {
			return f;
		}
		public void setF(Float f) {
			this.f = f;
		}
		@Override
		protected String doGet(Row line) {

			return null;
		}

	}

	/**
	 * Test class. Class which don't extend AbstractGetter class
	 * 
	 * @author Olivier Godineau
	 * 
	 */
	public static class CustomNoGetter {

		protected String doGet(List<String> line) {

			return null;
		}
	}

	/**
	 * loading test of SimpleGetter
	 * 
	 * @throws XPathExpressionException
	 */
	public void testSimpleGetter() throws XPathExpressionException {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil.openDocument(XML_SIMPLE_CONTENT);
			element = (Element) Util.evaluerDOM(doc, "/getter", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on Getter loading", e1);

		} catch (SAXException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML parsing", e1);

		} catch (IOException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML loading", e1);
		} catch (XPathExpressionException e) {
			fail(e.toString());
			LOGGER.debug("Error on XML parsing", e);
		}
		try {
			AbstractGetter getter = AbstractGetterLoader.getInstance().getGetter(element);

			assertTrue("Getter class differs from expected", getter.getClass().getSimpleName().equals("DefaultGetter"));
			assertEquals("Getter default value differs from expected", "defaut", getter.getDefaultValue());
			List<Cell> cells = new ArrayList<Cell>();
			cells.add(new Cell(0, "a"));
			cells.add(new Cell(1, "b"));
			cells.add(new Cell(2, "c"));
			Row row = new Row(1, cells, 3);

			// assertEquals(1, ((DefaultGetter)getter).getRang());
			assertEquals("getter value differs from expected", "b", getter.get(row));// Pour
																						// tester
																						// si
																						// on
																						// récupère
																						// bien
																						// le
																						// rang
																						// du
																						// defaultGetter
			assertNotNull("Getter filter expected", getter.getFilter());

		} catch (LoadException e) {
			fail(e.toString());
			LOGGER.debug("Error on Loading", e);

		}
	}

	/**
	 * Loading test of ConstantGetter
	 * 
	 * @throws XPathExpressionException
	 */
	public void testConstantGetter() throws XPathExpressionException {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil.openDocument(XML_CONSTANT_CONTENT);
			element = (Element) Util.evaluerDOM(doc, "/getter", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on Getter loading", e1);
		} catch (SAXException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML parsing", e1);
		} catch (IOException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML loading", e1);
		} catch (XPathExpressionException e) {
			fail(e.toString());
			LOGGER.debug("Error on XML parsing", e);
		}

		try {
			AbstractGetter getter = AbstractGetterLoader.getInstance().getGetter(element);

			assertTrue("Getter class differs from ConstantGetter",
					getter.getClass().getSimpleName().equals("ConstantGetter"));
			assertEquals("Getter default value differs from expected", "defaut", getter.getDefaultValue());
			assertNotNull("Getter filter expected", getter.getFilter());

		} catch (LoadException e) {
			fail(e.toString());
			LOGGER.debug("Error on Getter loading from XML", e);

		}
	}

	/**
	 * Loading test of CustomGetter
	 * 
	 * @throws XPathExpressionException
	 */
	public void testCustomGetter() throws XPathExpressionException {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil.openDocument(XML_CUSTOM_CONTENT);
			element = (Element) Util.evaluerDOM(doc, "/getter", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on Getter loading", e1);
		} catch (SAXException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML parsing", e1);
		} catch (IOException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML loading", e1);
		} catch (XPathExpressionException e) {
			fail(e.toString());
			LOGGER.debug("Error on XML parsing", e);
		}

		try {
			AbstractGetter getter = AbstractGetterLoader.getInstance().getGetter(element);

			assertTrue("Getter class differs from", getter.getClass().equals(CustomGetter.class));
			assertEquals("Getter default value differs from", "defaut", getter.getDefaultValue());
			assertNotNull("Filter expected", getter.getFilter());

			assertEquals("Getter property value differs from ", 127.075f, ((CustomGetter) getter).getF());

		} catch (LoadException e) {
			fail(e.toString());
			LOGGER.debug("Error on Getter loading", e);
		}
	}

	/**
	 * loading Test of class which don't extend AbstractGetter class.
	 */
	public void testFailedCustomGetter() {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil
					.openDocument("<getter><custom class=\"olg.csv.bean.loader.getter.AbstractGetterLoaderTestCase$CustomNoGetter\">"
							+ "								</custom>" + "							</getter>\n");
			element = (Element) Util.evaluerDOM(doc, "/getter", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on Getter loading", e1);
		} catch (SAXException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML parsing", e1);
		} catch (IOException e1) {
			fail(e1.toString());
			LOGGER.debug("Error on XML loading", e1);
		} catch (XPathExpressionException e) {
			fail(e.toString());
			LOGGER.debug("Error on XML parsing", e);
		}
		try {
			AbstractGetterLoader.getInstance().getGetter(element);
		} catch (XPathExpressionException e) {
			fail(e.toString());
			LOGGER.debug("Error on XML parsing", e);
		} catch (LoadException e) {
			assertEquals("Error message differs from", CustomNoGetter.class + " must extend " + AbstractGetter.class,
					e.getMessage());
			LOGGER.debug("Error on Getter loading", e);
		}
	}

}
