/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.loader.parser;

import java.io.IOException;
import java.util.Date;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;

import junit.framework.TestCase;
import olg.csv.bean.TestUtil;
import olg.csv.bean.loader.LoadException;
import olg.csv.bean.loader.Util;
import olg.csv.bean.parser.AbstractParser;
import olg.csv.bean.parser.ParseException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

/**
 * AbstractParserLoader class TestCase
 * 
 * @author Olivier Godineau
 * 
 */
public class AbstractParserLoaderTestCase extends TestCase {
	private static final Logger LOGGER = LoggerFactory.getLogger(AbstractParserLoaderTestCase.class);

	private static final String FLOAT = "127.075";

	private static final String STR = "pour un test";
	private static final String CUSTOM_PARSER_RETOUR = "test";
	private static final String XML_DATE_PARSER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" +

	"<parser><date format=\"dd-MM-yyyy\" /></parser>";

	private static final String XML_CUSTOM_PARSER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" + "<parser>"
			+ "	<custom class=\"olg.csv.bean.loader.parser.AbstractParserLoaderTestCase$CustomParser\">"
			+ "		<property name=\"str\">" + "			<value>" + STR + "</value>" + "		</property>"
			+ "		<property name=\"theFloat\">" + "			<value>" + FLOAT + "</value>" + "		</property>" + "	</custom>"
			+ "</parser>";

	/**
	 * Test class
	 * 
	 * @author Olivier Godineau
	 * 
	 */
	public static class CustomParser extends AbstractParser<String> {

		public CustomParser() {
			super();

		}
		private String str;
		private float theFloat;
		public String getStr() {
			return str;
		}
		public void setStr(String str) {
			this.str = str;
		}
		public float getTheFloat() {
			return theFloat;
		}
		public void setTheFloat(float aFloat) {
			this.theFloat = aFloat;
		}

		public String parse(String aString) throws ParseException {

			return CUSTOM_PARSER_RETOUR;
		}

	}

	/**
	 * Loading test of DateParser
	 */
	public void testDateParser() {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil.openDocument(XML_DATE_PARSER);
			element = (Element) Util.evaluerDOM(doc, "/parser", XPathConstants.NODE);

		} catch (ParserConfigurationException e1) {
			LOGGER.debug("Error on XML Parser configuration", e1);
			fail(e1.toString());

		} catch (SAXException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());

		} catch (IOException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());

		} catch (XPathExpressionException e) {
			LOGGER.debug("Error on XML Parsing", e);
			fail(e.toString());

		}

		try {
			AbstractParser<?> parser = AbstractParserLoader.getInstance().getParser(java.util.Date.class, element);
			assertNotNull("Parser loading failed", parser);
			Date date = (Date) parser.parse("01-04-2009");// Le parser est testé
			// dans
			// AbstractParserTestCase
			assertNotNull("Parsing failed ", date);
		} catch (XPathExpressionException e) {
			LOGGER.debug("Error on Parser loading", e);
			fail(e.toString());
		} catch (LoadException e) {
			LOGGER.debug("Error on Parser loading", e);
			fail(e.toString());

		} catch (ParseException e) {
			LOGGER.debug("Error on date parsing", e);
			fail(e.toString());

		}

	}

	/**
	 * Loading test of CustomParser
	 */
	public void testCustomParser() {
		Document doc = null;
		Element element = null;
		try {
			doc = TestUtil.openDocument(XML_CUSTOM_PARSER);
			element = (Element) Util.evaluerDOM(doc, "/parser", XPathConstants.NODE);
			AbstractParser<?> parser = AbstractParserLoader.getInstance().getParser(java.util.Date.class, element);
			assertNotNull("Parser Loading must return a not null parser ", parser);
			assertEquals("Parser Loading must return a CustomParser instance", CustomParser.class, parser.getClass());
			CustomParser concretParser = (CustomParser) parser;
			assertEquals("Setting customParser property failed", STR, concretParser.getStr());
			assertEquals("Setting customParser property failed", Float.valueOf(FLOAT), concretParser.getTheFloat());
			Object result = parser.parse("truc");
			assertNotNull("CustomParser parsing must return not null result", result);
			assertEquals("CustomParser parsing must return String instance", String.class, result.getClass());
			assertEquals("CustomParser parsing result differs from expected", CUSTOM_PARSER_RETOUR, result);

		} catch (ParserConfigurationException e1) {
			LOGGER.debug("Error on XML Parser configuration", e1);
			fail(e1.toString());
		} catch (SAXException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());
		} catch (IOException e1) {
			LOGGER.debug("Error on XML Parsing", e1);
			fail(e1.toString());
		} catch (XPathExpressionException e) {
			LOGGER.debug("Error on Parser loading", e);
			fail(e.toString());
		} catch (LoadException e) {
			LOGGER.debug("Error on Parser loading", e);
			fail(e.toString());
		} catch (ParseException e) {
			LOGGER.debug("Error on  Parsing", e);
			fail(e.toString());
		}

	}

}
