/*
 * Copyright 2012 Olivier Godineau
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package olg.csv.bean.parser;

import java.math.BigInteger;

import junit.framework.TestCase;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * PrimitiveType enum TestCase
 * 
 * @author Olivier Godineau
 * 
 */
public class PrimitiveTypeTestCase extends TestCase {
	private static final Logger LOGGER = LoggerFactory.getLogger(PrimitiveTypeTestCase.class);

	private final static String STR_TO_FAIL = "tralala";
	private final static String FLOAT_VALUE = "12.3";
	private final static String UNCONFORMED_FLOAT_VALUE = "12,3";
	private final static String NUMBER = "0123";

	public void testBoolean() {
		@SuppressWarnings("rawtypes")
		AbstractParser parser = PrimitiveType.booleanType.getParser();

		try {
			Boolean boo = (Boolean) parser.parse(STR_TO_FAIL);
			assertEquals("Boolean Parser must return  false if the string differs from true", (Boolean) false, boo);
		} catch (Exception e) {
			fail(e.getMessage());
		}

		try {
			boolean boo = (Boolean) parser.parse("true");
			assertEquals("Boolean Parser must parse 'true' string value", true, boo);
		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			boolean boo = (Boolean) parser.parse("false");
			assertEquals("Boolean Parser must return false with 'false' string", false, boo);
		} catch (Exception e) {
			fail(e.getMessage());
		}
		try {
			boolean boo = (Boolean) parser.parse(null);
			assertEquals("Boolean Parser must return false with null string", false, boo);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public void testByte() {
		@SuppressWarnings("rawtypes")
		AbstractParser parser = PrimitiveType.byteType.getParser();

		try {
			parser.parse(STR_TO_FAIL);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}

		try {
			parser.parse(FLOAT_VALUE);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(UNCONFORMED_FLOAT_VALUE);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			byte aByte = (Byte) parser.parse(NUMBER);
			assertEquals("Byte parser must parse byte string representation ", 123, aByte);
		} catch (Exception e) {

			fail(e.toString());
		}

		try {
			byte aByte = (Byte) parser.parse(null);
			assertEquals("Byte parser must return the default value ", 0, aByte);
		} catch (Exception e) {

			fail(e.toString());
		}
	}

	public void testShort() {
		@SuppressWarnings("rawtypes")
		AbstractParser parser = PrimitiveType.shortType.getParser();

		try {
			parser.parse(STR_TO_FAIL);
			fail();
		} catch (Exception ex) { // NOPMD by olivier on 10/01/12 01:07
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}

		try {
			parser.parse(FLOAT_VALUE);
			fail();
		} catch (Exception ex) { // NOPMD by olivier on 10/01/12 00:31
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(UNCONFORMED_FLOAT_VALUE);
			fail();
		} catch (Exception ex) { // NOPMD by olivier on 10/01/12 00:30

			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}
		try {
			short sht = (Short) parser.parse(NUMBER); // NOPMD by olivier on
														// 10/01/12 00:11
			assertEquals("Short parser must parse short String representation", 123, sht);
		} catch (Exception e) {
			LOGGER.error("Short parsing error", e);
			fail(e.toString());
		}

		try {
			short sht = (Short) parser.parse(null); // NOPMD by olivier on
													// 10/01/12 00:11
			assertEquals("Short parser must return the default value", 0, sht);
		} catch (Exception e) {
			LOGGER.error("Short parsing error", e);
			fail(e.toString());
		}

		try {
			parser.parse(Long.toString(Long.valueOf(Short.MAX_VALUE) + 1));
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(Long.toString(Long.valueOf(Short.MIN_VALUE) - 1));
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}

	}

	public void testInt() {
		AbstractParser<?> parser = PrimitiveType.intType.getParser();
		try {
			parser.parse(STR_TO_FAIL);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}

		try {
			parser.parse(FLOAT_VALUE);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(UNCONFORMED_FLOAT_VALUE);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			Integer integer = (Integer) parser.parse(NUMBER);
			assertEquals("Integer parser must return the expected integer value", Integer.valueOf(123), integer);
		} catch (Exception e) {
			LOGGER.error("Integer parsing error", e);
			fail(e.getMessage());
		}

		try {
			Integer integer = (Integer) parser.parse(null);
			assertEquals("Integer parser must return the default integer value", Integer.valueOf(0), integer);
		} catch (Exception e) {
			LOGGER.error("Integer parsing error", e);
			fail(e.getMessage());
		}

		try {
			parser.parse(Long.toString(Long.valueOf(Integer.MAX_VALUE) + 1));

			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(Long.toString(Long.valueOf(Integer.MIN_VALUE) - 1));
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}

	}

	public void testLong() {
		@SuppressWarnings("rawtypes")
		AbstractParser parser = PrimitiveType.longType.getParser();
		try {
			parser.parse(STR_TO_FAIL);

			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}

		try {
			parser.parse(FLOAT_VALUE);
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(UNCONFORMED_FLOAT_VALUE);
			fail();
		} catch (Exception ex) {

			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}
		try {
			Long aLong = (Long) parser.parse(NUMBER);
			assertEquals("long parser must return the expected long value", Long.valueOf(123), aLong);
		} catch (Exception e) {
			LOGGER.error("Long parsing error", e);
			fail(e.getMessage());
		}

		try {
			Long aLong = (Long) parser.parse(null);
			assertEquals("long parser must return the default long value", Long.valueOf(0), aLong);
		} catch (Exception e) {
			LOGGER.error("Long parsing error", e);
			fail(e.getMessage());
		}

		try {
			parser.parse(BigInteger.valueOf(Long.MAX_VALUE).add(BigInteger.valueOf(1)).toString());

			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}
		try {
			parser.parse(BigInteger.valueOf(Long.MIN_VALUE).add(BigInteger.valueOf(-1)).toString());
			fail();
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}

	}

	public void testFloat() {
		@SuppressWarnings("rawtypes")
		AbstractParser parser = PrimitiveType.floatType.getParser();
		try {
			parser.parse(STR_TO_FAIL);
			fail("Float parser must fail to parse a none float string representation");
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}

		try {
			Float aFloat = (Float) parser.parse(FLOAT_VALUE);
			assertEquals("Float parser must return the expected float value", Float.valueOf(12.3f), aFloat);

		} catch (Exception e) {
			LOGGER.error("Float parsing error", e);
			fail(e.toString());
		}
		try {
			parser.parse(UNCONFORMED_FLOAT_VALUE);
			fail("Float parser must fail to parse a none float string representation");
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}
		try {
			float aFloat = (Float) parser.parse(NUMBER);
			assertEquals("Float parser must return the expected float value", 123f, aFloat);
		} catch (Exception e) {
			LOGGER.error("Float parsing error", e);
			fail(e.toString());
		}

		try {
			float aFloat = (Float) parser.parse(null);
			assertEquals("Float parser must return the default value", 0f, aFloat);
		} catch (Exception e) {
			LOGGER.error("Float parsing error", e);
			fail(e.toString());
		}

	}

	public void testDouble() {

		AbstractParser<?> parser = PrimitiveType.doubleType.getParser();

		try {
			parser.parse(STR_TO_FAIL);
			fail("Double parser must fail to parse a none float string representation");
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());

		}

		try {
			Double aDouble = (Double) parser.parse(FLOAT_VALUE);
			assertEquals("Double parser must return the expected float value", Double.valueOf(12.3), aDouble);

		} catch (Exception e) {
			LOGGER.error("Float parsing error", e);
			fail(e.toString());
		}
		try {
			parser.parse(UNCONFORMED_FLOAT_VALUE);
			fail("Double parser must fail to parse a none float string representation");
		} catch (Exception ex) {
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}
		try {
			Double aDouble = (Double) parser.parse(NUMBER);
			assertEquals("Double parser must return the expected float value", Double.valueOf(123), aDouble);
		} catch (Exception e) {
			LOGGER.error("aDouble parsing error", e);
			fail(e.toString());
		}

		try {
			Double aDouble = (Double) parser.parse(null);
			assertEquals("Double parser must return the default value", Double.valueOf(0), aDouble);
		} catch (Exception e) {
			LOGGER.error("Float parsing error", e);
			fail(e.toString());
		}

	}

	public void testChar() {

		AbstractParser<?> parser = PrimitiveType.charType.getParser();
		try {
			parser.parse(STR_TO_FAIL);
			fail("Char parser must fail to parse a none char string representation");
		} catch (Exception ex) { // NOPMD by olivier on 10/01/12 00:32
			assertEquals("ParseException must be the Exception thrown on parse error", "ParseException", ex.getClass()
					.getSimpleName());
		}

		try {
			char aChar = (Character) parser.parse("e");
			assertEquals("Char parser must return the expected char value", 'e', aChar);
		} catch (Exception e) {
			LOGGER.error("Char parsing error", e);
			fail(e.toString());
		}

	}
}
